import socket

import eventlet
from eventlet.event import Event
from kombu import Queue, Exchange, Connection
from kombu.exceptions import TimeoutError
from mock import patch, Mock, ANY
import pytest

from nameko.messaging import QueueConsumer, AMQP_URI_CONFIG_KEY
from nameko.testing.utils import assert_stops_raising, get_rabbit_connections


TIMEOUT = 5

exchange = Exchange('spam')
ham_queue = Queue('ham', exchange=exchange, auto_delete=False)


class MessageHandler(object):
    queue = ham_queue

    def __init__(self):
        self.handle_message_called = Event()

    def handle_message(self, body, message):
        self.handle_message_called.send(message)

    def wait(self):
        return self.handle_message_called.wait()


def spawn_thread(method, protected):
    return eventlet.spawn(method)


def test_lifecycle(rabbit_manager, rabbit_config):

    container = Mock()
    container.config = rabbit_config
    container.max_workers = 3
    container.spawn_managed_thread.side_effect = spawn_thread

    queue_consumer = QueueConsumer()
    queue_consumer.bind("queue_consumer", container)

    handler = MessageHandler()

    queue_consumer.register_provider(handler)

    queue_consumer.start()

    # making sure the QueueConsumer uses the container to spawn threads
    container.spawn_managed_thread.assert_called_once_with(ANY, protected=True)

    vhost = rabbit_config['vhost']
    rabbit_manager.publish(vhost, 'spam', '', 'shrub')

    message = handler.wait()

    gt = eventlet.spawn(queue_consumer.unregister_provider, handler)

    # wait for the handler to be removed
    with eventlet.Timeout(TIMEOUT):
        while len(queue_consumer._consumers):
            eventlet.sleep()

    # remove_consumer has to wait for all messages to be acked
    assert not gt.dead

    # the consumer should have stopped and not accept any new messages
    rabbit_manager.publish(vhost, 'spam', '', 'ni')

    # this should cause the consumer to finish shutting down
    queue_consumer.ack_message(message)
    with eventlet.Timeout(TIMEOUT):
        gt.wait()

    # there should be a message left on the queue
    messages = rabbit_manager.get_messages(vhost, 'ham')
    assert ['ni'] == [msg['payload'] for msg in messages]

    queue_consumer.kill()


def test_reentrant_start_stops(rabbit_config):
    container = Mock()
    container.config = rabbit_config
    container.max_workers = 3
    container.spawn_managed_thread = spawn_thread

    queue_consumer = QueueConsumer()
    queue_consumer.bind("queue_consumer", container)

    queue_consumer.start()
    gt = queue_consumer._gt

    # nothing should happen as the consumer has already been started
    queue_consumer.start()
    assert gt is queue_consumer._gt

    queue_consumer.kill()


def test_stop_while_starting():
    started = Event()

    container = Mock()
    container.config = {AMQP_URI_CONFIG_KEY: None}
    container.max_workers = 3
    container.spawn_managed_thread = spawn_thread

    class BrokenConnConsumer(QueueConsumer):
        def consume(self, *args, **kwargs):
            started.send(None)
            # kombu will retry again and again on broken connections
            # so we have to make sure the event is reset to allow consume
            # to be called again
            started.reset()
            return super(BrokenConnConsumer, self).consume(*args, **kwargs)

    queue_consumer = BrokenConnConsumer()
    queue_consumer.bind("queue_consumer", container)

    handler = MessageHandler()
    queue_consumer.register_provider(handler)

    with eventlet.Timeout(TIMEOUT):
        with patch.object(Connection, 'connect', autospec=True) as connect:
            # patch connection to raise an error
            connect.side_effect = TimeoutError('test')
            # try to start the queue consumer
            gt = eventlet.spawn(queue_consumer.start)
            # wait for the queue consumer to begin starting and
            # then immediately stop it
            started.wait()

    with eventlet.Timeout(TIMEOUT):
        queue_consumer.unregister_provider(handler)
        queue_consumer.stop()

    with eventlet.Timeout(TIMEOUT):
        # we expect the queue_consumer.start thread to finish
        # almost immediately adn when it does the queue_consumer thread
        # should be dead too
        while not gt.dead:
            eventlet.sleep()

        assert queue_consumer._gt.dead


def test_error_stops_consumer_thread():
    container = Mock()
    container.config = {AMQP_URI_CONFIG_KEY: None}
    container.max_workers = 3
    container.spawn_managed_thread = spawn_thread

    queue_consumer = QueueConsumer()
    queue_consumer.bind("queue_consumer", container)

    handler = MessageHandler()
    queue_consumer.register_provider(handler)

    with eventlet.Timeout(TIMEOUT):
        with patch.object(
                Connection, 'drain_events', autospec=True) as drain_events:
            drain_events.side_effect = Exception('test')
            queue_consumer.start()

    with pytest.raises(Exception) as exc_info:
        queue_consumer._gt.wait()

    assert exc_info.value.args == ('test',)


def test_on_consume_error_kills_consumer():
    container = Mock()
    container.config = {AMQP_URI_CONFIG_KEY: None}
    container.max_workers = 1
    container.spawn_managed_thread = spawn_thread

    queue_consumer = QueueConsumer()

    queue_consumer.bind("queue_consumer", container)

    handler = MessageHandler()
    queue_consumer.register_provider(handler)

    with patch.object(queue_consumer, 'on_consume_ready') as on_consume_ready:
        on_consume_ready.side_effect = Exception('err')
        queue_consumer.start()

        with pytest.raises(Exception):
            queue_consumer._gt.wait()


def test_reconnect_on_socket_error():

    container = Mock()
    container.config = {AMQP_URI_CONFIG_KEY: None}
    container.max_workers = 1
    container.spawn_managed_thread = spawn_thread

    connection_revived = Mock()

    queue_consumer = QueueConsumer()
    queue_consumer.on_connection_revived = connection_revived

    queue_consumer.bind("queue_consumer", container)

    handler = MessageHandler()
    queue_consumer.register_provider(handler)
    queue_consumer.start()

    with patch.object(
            Connection, 'drain_events', autospec=True) as drain_events:
        drain_events.side_effect = socket.error('test-error')

        def check_reconnected():
            assert connection_revived.call_count > 1
        assert_stops_raising(check_reconnected)


def test_prefetch_count(rabbit_manager, rabbit_config):
    container = Mock()
    container.config = rabbit_config
    container.max_workers = 1
    container.spawn_managed_thread = spawn_thread

    queue_consumer1 = QueueConsumer()
    queue_consumer1.bind("queue_consumer", container)

    queue_consumer2 = QueueConsumer()
    queue_consumer2.bind("queue_consumer", container)

    consumer_continue = Event()

    class Handler1(object):
        queue = ham_queue

        def handle_message(self, body, message):
            consumer_continue.wait()
            queue_consumer1.ack_message(message)

    messages = []

    class Handler2(object):
        queue = ham_queue

        def handle_message(self, body, message):
            messages.append(body)
            queue_consumer2.ack_message(message)

    handler1 = Handler1()
    handler2 = Handler2()

    queue_consumer1.register_provider(handler1)
    queue_consumer2.register_provider(handler2)

    queue_consumer1.start()
    queue_consumer2.start()

    vhost = rabbit_config['vhost']
    # the first consumer only has a prefetch_count of 1 and will only
    # consume 1 message and wait in handler1()
    rabbit_manager.publish(vhost, 'spam', '', 'ham')
    # the next message will go to handler2() no matter of any prefetch_count
    rabbit_manager.publish(vhost, 'spam', '', 'eggs')
    # the third message is only going to handler2 because the first consumer
    # has a prefetch_count of 1 and thus is unable to deal with another message
    # until having ACKed the first one
    rabbit_manager.publish(vhost, 'spam', '', 'bacon')

    with eventlet.Timeout(TIMEOUT):
        while len(messages) < 2:
            eventlet.sleep()

    # allow the waiting consumer to ack its message
    consumer_continue.send(None)

    assert messages == ['eggs', 'bacon']

    queue_consumer1.unregister_provider(handler1)
    queue_consumer2.unregister_provider(handler2)

    queue_consumer1.kill()
    queue_consumer2.kill()


def test_kill_closes_connections(rabbit_manager, rabbit_config):

    container = Mock()
    container.config = rabbit_config
    container.max_workers = 1
    container.spawn_managed_thread = spawn_thread

    queue_consumer = QueueConsumer()
    queue_consumer.bind("queue_consumer", container)

    class Handler(object):
        queue = ham_queue

        def handle_message(self, body, message):
            pass

    queue_consumer.register_provider(Handler())
    queue_consumer.start()

    # kill should close all connections
    queue_consumer.kill()

    # no connections should remain for our vhost
    vhost = rabbit_config['vhost']
    connections = get_rabbit_connections(vhost, rabbit_manager)
    if connections:
        for connection in connections:
            assert connection['vhost'] != vhost



from mock import patch
import pytest
import socket

from nameko.containers import WorkerContext
from nameko.dependencies import injection, InjectionProvider, DependencyFactory
from nameko.exceptions import RemoteError
from nameko.rpc import rpc
from nameko.standalone.rpc import RpcProxy


class ContextReader(InjectionProvider):
    """ Access values from the worker context data.

    This is a test facilty! Write specific InjectionProviders to make use of
    values in ``WorkerContext.data``, don't expose it directly.
    """
    def acquire_injection(self, worker_ctx):
        def get_context_value(key):
            return worker_ctx.data.get(key)
        return get_context_value


@injection
def context_reader():
    return DependencyFactory(ContextReader)


class FooService(object):
    name = 'foobar'

    get_context_value = context_reader()

    @rpc
    def spam(self, ham):
        return ham

    @rpc
    def broken(self):
        raise ExampleError('broken')

    @rpc
    def get_context_data(self, name):
        return self.get_context_value(name)


class ExampleError(Exception):
    pass


class CustomWorkerContext(WorkerContext):
    context_keys = ("custom_header",)


def test_proxy(container_factory, rabbit_config):

    container = container_factory(FooService, rabbit_config)
    container.start()

    with RpcProxy('foobar', rabbit_config) as foo:
        assert foo.spam(ham='eggs') == 'eggs'
        assert foo.spam(ham='eggs') == 'eggs'  # test re-use


def test_proxy_manual_start_stop(container_factory, rabbit_config):

    container = container_factory(FooService, rabbit_config)
    container.start()

    foobar_proxy = RpcProxy('foobar', rabbit_config)
    foo = foobar_proxy.start()
    assert foo.spam(ham='eggs') == 'eggs'
    assert foo.spam(ham='eggs') == 'eggs'  # test re-use
    foobar_proxy.stop()


def test_proxy_context_data(container_factory, rabbit_config):

    container = container_factory(FooService, rabbit_config)
    container.start()

    context_data = {'language': 'en'}
    with RpcProxy('foobar', rabbit_config, context_data) as foo:
        assert foo.get_context_data('language') == 'en'

    context_data = {'language': 'fr'}
    with RpcProxy('foobar', rabbit_config, context_data) as foo:
        assert foo.get_context_data('language') == 'fr'


def test_proxy_worker_context(container_factory, rabbit_config):

    container = container_factory(FooService, rabbit_config,
                                  CustomWorkerContext)
    container.start()

    context_data = {'custom_header': 'custom_value'}

    with RpcProxy('foobar', rabbit_config, context_data,
                  CustomWorkerContext) as foo:
        assert foo.get_context_data('custom_header') == "custom_value"

    with RpcProxy('foobar', rabbit_config, context_data) as foo:
        assert foo.get_context_data('custom_header') is None


def test_proxy_remote_error(container_factory, rabbit_config):

    container = container_factory(FooService, rabbit_config)
    container.start()

    with RpcProxy("foobar", rabbit_config) as proxy:
        with pytest.raises(RemoteError) as exc_info:
            proxy.broken()
        assert exc_info.value.exc_type == "ExampleError"


def test_proxy_connection_error(container_factory, rabbit_config):

    container = container_factory(FooService, rabbit_config)
    container.start()

    path = 'nameko.standalone.rpc.PollingQueueConsumer.poll_messages'
    with patch(path, autospec=True) as poll_messages:
        poll_messages.side_effect = socket.error

        with RpcProxy("foobar", rabbit_config) as proxy:
            with pytest.raises(socket.error):
                proxy.spam("")



import inspect


class DependencyNotFound(AttributeError):
    pass


class RpcTimeout(Exception):
    pass


class ContainerBeingKilled(Exception):
    """Raised by :meth:`Container.spawn_worker` if it has started a ``kill``
    sequence.

    Entrypoint providers should catch this and react as if they hadn't been
    available in the first place, e.g. an rpc consumer should probably requeue
    the message.

    We need this because eventlet may yield during the execution of
    :meth:`Container.kill`, giving entrypoints a chance to fire before
    they themselves have been killed.
    """


registry = {}


def get_module_path(exc_type):
    """ Return the dotted module path of `exc_type`, including the class name.

    e.g.::

        >>> get_module_path(MethodNotFound)
        >>> "nameko.exceptions.MethodNotFound"

    """
    module = inspect.getmodule(exc_type)
    return "{}.{}".format(module.__name__, exc_type.__name__)


class RemoteError(Exception):
    """ Exception to raise at the caller if an exception occured in the
    remote worker.
    """
    def __init__(self, exc_type=None, value=""):
        self.exc_type = exc_type
        self.value = value
        message = '{} {}'.format(exc_type, value)
        super(RemoteError, self).__init__(message)


def serialize(exc):
    """ Serialize `self.exc` into a data dictionary representing it.
    """
    try:
        value = unicode(exc)
    except Exception:
        value = '[__unicode__ failed]'

    return {
        'exc_type': type(exc).__name__,
        'exc_path': get_module_path(type(exc)),
        'exc_args': exc.args,
        'value': value,
    }


def deserialize(data):
    """ Deserialize `data` to an exception instance.

    If the `exc_path` value matches an exception registered as
    ``deserializable``, return an instance of that exception type.
    Otherwise, return a `RemoteError` instance describing the exception
    that occured.
    """
    key = data.get('exc_path')
    if key in registry:
        exc_args = data.get('exc_args', ())
        return registry[key](*exc_args)

    exc_type = data.get('exc_type')
    value = data.get('value')
    return RemoteError(exc_type=exc_type, value=value)


def deserialize_to_instance(exc_type):
    """ Decorator that registers `exc_type` as deserializable back into an
    instance, rather than a :class:`RemoteError`. See :func:`deserialize`.
    """
    key = get_module_path(exc_type)
    registry[key] = exc_type
    return exc_type


@deserialize_to_instance
class MethodNotFound(Exception):
    pass


@deserialize_to_instance
class IncorrectSignature(Exception):
    pass


class UnknownService(Exception):
    def __init__(self, service_name):
        self._service_name = service_name
        super(UnknownService, self).__init__(service_name)

    def __str__(self):
        return "Unknown service `{}`".format(self._service_name)


class UnserializableValueError(Exception):
    def __init__(self, value):
        try:
            self.repr_value = repr(value)
        except Exception:
            self.repr_value = '[__repr__ failed]'
        super(UnserializableValueError, self).__init__()

    def __str__(self):
        return "Unserializable value: `{}`".format(self.repr_value)



import eventlet
from mock import Mock, patch
import pytest

from nameko.dependencies import (
    entrypoint, EntrypointProvider, prepare_entrypoint_providers,
    injection, InjectionProvider, prepare_injection_providers,
    DependencyFactory, DependencyTypeError, dependency,
    DependencyProvider, PROCESS_SHARED, CONTAINER_SHARED, ProviderCollector)
from nameko.containers import ServiceContainer, WorkerContext


class SharedProvider(DependencyProvider):
    pass


class NestedProvider(DependencyProvider):
    pass


@dependency
def shared_provider(*args, **kwargs):
    return DependencyFactory(SharedProvider, *args, **kwargs)


@dependency
def nested_provider(*args, **kwargs):
    return DependencyFactory(NestedProvider, *args, **kwargs)


class FooProvider(EntrypointProvider):

    shared_provider = shared_provider(shared=CONTAINER_SHARED)

    def __init__(self, *args, **kwargs):
        self.args = args
        self.kwargs = kwargs


@entrypoint
def foobar(*args, **kwargs):
    """foobar-doc"""
    return DependencyFactory(FooProvider, *args, **kwargs)


class BarProvider(InjectionProvider):

    nested_provider = nested_provider()
    shared_provider = shared_provider(shared=CONTAINER_SHARED)

    def __init__(self, *args, **kwargs):
        self.args = args
        self.kwargs = kwargs

    def acquire_injection(self):
        return lambda *args, **kwargs: "bar"


@injection
def barfoo(*args, **kwargs):
    return DependencyFactory(BarProvider, *args, **kwargs)


class ExampleService(object):

    injected = barfoo("arg", kwarg="kwarg")

    @foobar("arg", kwarg="kwarg")
    def echo(self, value):
        return value


def test_dependency_decorator():
    # make sure foobar is properly wrapped
    assert foobar.__doc__ == 'foobar-doc'
    assert foobar.func_name == 'foobar'

    def foo(spam):
        pass

    decorated_foo = foobar(foo='bar')(foo)

    # make sure dependency_deocorator passes through the decorated method
    assert decorated_foo is foo


def test_prepare_entrypoint_providers():

    config = Mock()
    container = ServiceContainer(ExampleService, WorkerContext, config)

    providers = list(prepare_entrypoint_providers(container))
    assert len(providers) == 1
    provider = providers[0]

    assert provider.name == "echo"
    assert isinstance(provider, FooProvider)
    assert provider.args == ("arg",)
    assert provider.kwargs == {"kwarg": "kwarg"}

    including_nested = list(prepare_entrypoint_providers(
        container, include_dependencies=True))
    assert len(including_nested) == 2

    assert set([type(dep) for dep in including_nested]) == set(
        [FooProvider, SharedProvider])


def test_prepare_injection_providers():

    config = Mock()
    container = ServiceContainer(ExampleService, WorkerContext, config)

    providers = list(prepare_injection_providers(container))
    assert len(providers) == 1
    provider = providers[0]

    assert provider.name == "injected"
    assert isinstance(provider, BarProvider)
    assert provider.args == ("arg",)
    assert provider.kwargs == {"kwarg": "kwarg"}

    including_nested = list(prepare_injection_providers(
        container, include_dependencies=True))
    assert len(including_nested) == 3

    assert set([type(dep) for dep in including_nested]) == set(
        [BarProvider, SharedProvider, NestedProvider])


def test_nested_dependencies(rabbit_config):

    container = Mock()
    container.config = rabbit_config

    bar_factory = DependencyFactory(BarProvider)
    bar = bar_factory.create_and_bind_instance("bar", container)

    dependencies = list(bar.nested_dependencies)
    assert len(dependencies) == 2
    assert dependencies[0].container == dependencies[1].container == container
    assert set([type(dep) for dep in dependencies]) == set([SharedProvider,
                                                           NestedProvider])


def test_dependency_instances_are_shared(container_factory, rabbit_config):

    example_providers = []

    class ExampleProvider(InjectionProvider):
        """ Example provider with two shared sub-dependencies - one at shared
        at the container level, one at the process level.
        """
        ct_shared = shared_provider(shared=CONTAINER_SHARED)
        process_shared = shared_provider(shared=PROCESS_SHARED)

        def __init__(self):
            # track ExampleProvider instances
            example_providers.append(self)

        def acquire_injection(self):
            pass

    @injection
    def injection_provider():
        return DependencyFactory(ExampleProvider)

    class Service():
        provider1 = injection_provider()
        provider2 = injection_provider()

    with patch('nameko.dependencies.shared_dependencies', {}) as shared_deps:
        container1 = container_factory(Service, rabbit_config)
        container2 = container_factory(Service, rabbit_config)

    # four shared providers should exist (two for each service)
    assert len(example_providers) == 4

    # but all four share their sub-dependencies
    # two containers in one process result in three sharing_keys
    assert set(shared_deps.keys()) == set([container1, container2,
                                           PROCESS_SHARED])

    # exactly three shared dependencies should have been instantiated
    all_dependencies = []
    for shared in shared_deps.values():
        all_dependencies.extend(shared.values())
    assert len(all_dependencies) == 3
    assert all(isinstance(dep, SharedProvider) for dep in all_dependencies)


def test_entrypoint_decorator_does_not_mutate_service():
    service = ExampleService()
    assert service.echo(1) == 1


def test_decorated_functions_must_return_dependency_factories():

    with pytest.raises(DependencyTypeError):
        @dependency
        def meth():
            pass
        meth()

    with pytest.raises(DependencyTypeError):
        @injection
        def foo():
            pass
        foo()

    with pytest.raises(DependencyTypeError):
        @entrypoint
        def bar():
            pass

        @bar
        def baz():
            pass


def test_stringify():
    dep = DependencyProvider()
    assert str(dep).startswith('<DependencyProvider [unbound] at')

    container = Mock()
    container.service_name = 'foobar'
    dep.bind('spam', container)
    assert str(dep).startswith('<DependencyProvider [foobar.spam] at')


def test_provider_collector():
    collector = ProviderCollector()

    provider1 = object()
    provider2 = object()
    collector.register_provider(provider1)
    collector.register_provider(provider2)

    assert provider1 in collector._providers
    assert provider2 in collector._providers

    collector.unregister_provider(provider1)
    assert provider1 not in collector._providers

    # unregister missing provider is a no-op
    collector.unregister_provider(provider1)

    # stop() should block while a provider remains
    with pytest.raises(eventlet.Timeout):
        with eventlet.Timeout(0):
            collector.stop()

    # stop() will return when the final provider is unregistered
    with eventlet.Timeout(0):
        collector.unregister_provider(provider2)
        collector.stop()



from collections import namedtuple
from path import path

SERVICE_INDEX_FILENAME = 'index.rst'

LEVEL_UNDERLINES = '=-~^'


def indent(text, size=4):
    lines = text.split('\n')
    indent = ' ' * size
    indented = '\n'.join(
        indent + line
        for line in lines
    )
    return indented


class RstPagePrinter(object):
    def __init__(self, output):
        self.output = output

        self.pages = []

    def __enter__(self):
        return self

    def __exit__(self, exc_type, exc_val, exc_tb):
        self.flush()

    def add_page(self, page):
        """ Register a renderer page with this collection """
        self.pages.append(page)

    def _file_path(self, file_name):
        return str(self.output_path.joinpath(file_name))

    @property
    def output_path(self):
        return path(self.output)

    @property
    def sorted_pages(self):
        return sorted(self.pages, key=lambda p: p.title)

    def flush(self):
        """ Flush all captured pages to output """
        if not self.pages:
            return

        # index first
        with open(self._file_path(SERVICE_INDEX_FILENAME), 'w') as f:
            f.write(self._index_contents())

        for page in self.pages:
            with open(self._file_path(page.filename), 'w') as f:
                f.write(page.content)

    def _index_contents(self):
        page_lines = indent('\n'.join(
            page.title for page in self.sorted_pages))
        lines = [
            'Services',
            '========',
            '',
            '.. toctree::',
            '    :maxdepth: 3',
            '',
            page_lines
        ]
        return '\n'.join(lines)


RenderedPage = namedtuple('RenderedPage', ['filename', 'title', 'content'])


def render_page(name, parts):
    """ Create a rendered page with a given name and a list of rendered
    contents """
    return RenderedPage(
        filename='{}.rst'.format(name),
        title=name,
        content='\n'.join(parts)
    )


def render_include_module(path):
    """ Render an instruction to include documentation for a module """
    return '.. automodule:: {}\n'.format(path)


def render_see_also_section(contents):
    """ Render a 'See Also' section with the given contents """
    return '.. seealso::\n\n{}\n'.format(
        indent('\n'.join(contents))
    )


def render_definition_list(contents):
    """ Render a definition list with the given items """
    return '{}\n'.format('\n'.join(contents))


def render_definition(term, description):
    """ Render a single definition with the given term and definition """
    return '{}\n{}'.format(term, indent(description))


def render_section(contents):
    """ Render an abstract section with the given contents """
    return '\n'.join(contents)


def render_include_method(path, no_index=False, extras=None):
    """ Render an instruction to include documentation for a method """
    extras = extras or []

    extra_lines = []
    if no_index:
        extra_lines.append(':noindex:')

    if extras:
        extra_lines.append('')

    for extra in extras:
        extra_lines.append(extra)

    extra_text = '\n'.join(extra_lines)
    extra_indented_content = indent(extra_text)

    return '.. automethod:: {}\n{}\n'.format(path, extra_indented_content)


def render_title(text, level=1, as_code=False):
    """ Render heading text at the given level (lower numbers more
    prominent), optional presenting it as code/monospaced """
    underline_char = LEVEL_UNDERLINES[level - 1]

    title_text = text if not as_code else '``{}``'.format(text)
    underline = underline_char * len(title_text)

    return '{}\n{}\n'.format(title_text, underline)


def render_class_reference(path):
    """ Render a cross-reference to a class """
    return ':class:`{}`'.format(path)


def render_instruction(name, content):
    """ Render an arbitrary in-line instruction with the given name and
    content """
    return ':{}: {}'.format(name, content)


def render_include_class(path, no_index=False, extras=None):
    """ Render an instruction to include documentation for a class """
    extras = extras or []

    extra_lines = []
    if no_index:
        extra_lines.append(':noindex:')

    if extras:
        extra_lines.append('')

    for extra in extras:
        extra_lines.append(extra)

    extra_text = '\n'.join(extra_lines)
    extra_indented_content = indent(extra_text)

    return '.. autoclass:: {}\n{}\n'.format(path, extra_indented_content)



import tempfile

from kombu import Exchange, Queue
from mock import Mock
from sqlalchemy import Column, Integer, String, create_engine
from sqlalchemy.ext.declarative import declarative_base

from nameko.contrib.sqlalchemy import orm_session, ORM_DB_URIS_KEY
from nameko.events import event_handler, event_dispatcher, Event
from nameko.messaging import publisher, consume
from nameko.rpc import rpc, rpc_proxy
from nameko.timer import timer
from nameko.testing.utils import wait_for_call

DeclBase = declarative_base(name='foo_base')


class FooModel(DeclBase):
    __tablename__ = 'spam'
    id = Column(Integer, primary_key=True)
    data = Column(String)


class SpamEvent(Event):
    type = 'spam'


foobar_ex = Exchange('foobar_ex', durable=False)
foobar_queue = Queue('foobar_queue', exchange=foobar_ex, durable=False)


class FooService(object):
    name = 'foo-service'

    foo_session = orm_session(DeclBase)
    dispatch_event = event_dispatcher()
    foo_service = rpc_proxy('foo-service')
    publish = publisher(queue=foobar_queue)

    @timer(interval=1)
    def handle_timer(self):
        ham = 'ham'
        self.dispatch_event(SpamEvent(ham))

    @event_handler('foo-service', 'spam')
    def handle_spam(self, evt_data):
        ham = self.foo_service.spam(evt_data)
        handle_spam_called(ham)

    @rpc
    def spam(self, ham):
        self.publish("message")
        self.foo_session.add(FooModel(data=ham))
        self.foo_session.commit()
        self.foo_session.flush()
        return ham + ' & eggs'

    @consume(queue=foobar_queue)
    def foo(self, msg):
        handle_foo_called(msg)


handle_spam_called = Mock()
handle_foo_called = Mock()


def test_example_service(container_factory, rabbit_config):

    db_uri = 'sqlite:///{}'.format(tempfile.NamedTemporaryFile().name)
    engine = create_engine(db_uri)
    FooModel.metadata.create_all(engine)

    config = {
        ORM_DB_URIS_KEY: {
            'foo-service:foo_base': db_uri
        }
    }
    config.update(rabbit_config)

    container = container_factory(FooService, config)
    container.start()

    with wait_for_call(5, handle_spam_called) as handle_spam:
        handle_spam.assert_called_with('ham & eggs')

    with wait_for_call(5, handle_foo_called) as handle_foo:
        handle_foo.assert_called_with('message')

    entries = list(engine.execute('SELECT data FROM spam LIMIT 1'))
    assert entries == [('ham',)]

    container.stop()



"""
Utilities for testing nameko services.
"""

from collections import OrderedDict
from contextlib import contextmanager
import inspect

from eventlet import event
from mock import MagicMock

from nameko.dependencies import (
    DependencyFactory, InjectionProvider, EntrypointProvider, entrypoint)
from nameko.exceptions import DependencyNotFound


@contextmanager
def entrypoint_hook(container, name, context_data=None):
    """ Yield a function providing an entrypoint into a hosted service.

    The yielded function may be called as if it were the bare method defined
    in the service class. Intended to be used as an integration testing
    utility.

    **Usage**

    To verify that ServiceX and ServiceY are compatible, make an integration
    test that checks their interaction:

    .. literalinclude:: examples/testing/integration_test.py

    """
    provider = next((entrypoint for entrypoint in container.entrypoints
                     if entrypoint.name == name), None)

    if provider is None:
        raise DependencyNotFound("No entrypoint called '{}' found "
                                 "on container {}.".format(name, container))

    def hook(*args, **kwargs):
        result = event.Event()

        def handle_result(worker_ctx, res=None, exc_info=None):
            result.send(res, exc_info)
            return res, exc_info

        container.spawn_worker(provider, args, kwargs,
                               context_data=context_data,
                               handle_result=handle_result)
        return result.wait()

    yield hook


def worker_factory(service_cls, **injections):
    """ Return an instance of ``service_cls`` with its injected dependencies
    replaced with Mock objects, or as given in ``injections``.

    **Usage**

    The following example service proxies calls to a "math" service via
    and ``rpc_proxy`` injection::

        from nameko.rpc import rpc_proxy, rpc

        class ConversionService(object):
            math = rpc_proxy("math_service")

            @rpc
            def inches_to_cm(self, inches):
                return self.math.multiply(inches, 2.54)

            @rpc
            def cm_to_inches(self, cms):
                return self.math.divide(cms, 2.54)

    Use the ``worker_factory`` to create an unhosted instance of
    ``ConversionService`` with its injections replaced by Mock objects::

        service = worker_factory(ConversionService)

    Nameko's entrypoints do not modify the service methods, so they can be
    called directly on an unhosted instance. The injection Mocks can be used
    as any other Mock object, so a complete unit test for Service may look
    like this::

        # create worker instance
        service = worker_factory(Service)

        # replace "math" service
        service.math.multiply.side_effect = lambda x, y: x * y
        service.math.divide.side_effect = lambda x, y: x / y

        # test inches_to_cm business logic
        assert service.inches_to_cm(300) == 762
        service.math.multiply.assert_called_once_with(300, 2.54)

        # test cms_to_inches business logic
        assert service.cms_to_inches(762) == 300
        service.math.divide.assert_called_once_with(762, 2.54)

    *Providing Injections*

    The ``**injections`` kwargs to ``worker_factory`` can be used to provide
    a replacement injection instead of a Mock. For example, to unit test a
    service against a real database:

    .. literalinclude:: examples/testing/unit_with_provided_injection_test.py

    """
    service = service_cls()
    for name, attr in inspect.getmembers(service_cls):
        if isinstance(attr, DependencyFactory):
            factory = attr
            if issubclass(factory.dep_cls, InjectionProvider):
                try:
                    injection = injections[name]
                except KeyError:
                    injection = MagicMock()
                setattr(service, name, injection)
    return service


class MockInjection(InjectionProvider):
    def __init__(self, name):
        self.name = name
        self.injection = MagicMock()

    def acquire_injection(self, worker_ctx):
        return self.injection


def replace_injections(container, *injections):
    """ Replace the injections on ``container`` with :class:`MockInjection`
    objects if they are named in ``injections``.

    Return the :attr:`MockInjection.injection` of the replacements, so that
    calls to the replaced injections can be inspected. Return a single object
    if only one injection was replaced, and a generator yielding the
    replacements in the same order as ``names`` otherwise.

    Replacements are made on the container instance and have no effect on the
    service class. New container instances are therefore unaffected by
    replacements on previous instances.

    **Usage**

    ::

        from nameko.rpc import rpc_proxy, rpc
        from nameko.standalone.rpc import RpcProxy

        class ConversionService(object):
            math = rpc_proxy("math_service")

            @rpc
            def inches_to_cm(self, inches):
                return self.math.multiply(inches, 2.54)

            @rpc
            def cm_to_inches(self, cms):
                return self.math.divide(cms, 2.54)

        container = ServiceContainer(ConversionService, config)
        math = replace_injections(container, "math")

        container.start()

        with RpcProxy('conversionservice', config) as proxy:
            proxy.cm_to_inches(100)

        # assert that the injection was called as expected
        math.divide.assert_called_once_with(100, 2.54)

    """
    if container.started:
        raise RuntimeError('You must replace injections before the '
                           'container is started.')

    injection_deps = list(container.injections)
    injection_names = {dep.name for dep in injection_deps}

    missing = set(injections) - injection_names
    if missing:
        raise DependencyNotFound("Injections(s) '{}' not found on {}.".format(
            missing, container))

    replacements = OrderedDict()

    named_injections = {dep.name: dep for dep in container.injections
                        if dep.name in injections}
    for name in injections:
        dependency = named_injections[name]
        replacement = MockInjection(name)
        replacements[dependency] = replacement
        container.dependencies.remove(dependency)
        container.dependencies.add(replacement)

    # if only one name was provided, return any replacement directly
    # otherwise return a generator
    res = (replacement.injection for replacement in replacements.values())
    if len(injections) == 1:
        return next(res)
    return res


def restrict_entrypoints(container, *entrypoints):
    """ Restrict the entrypoints on ``container`` to those named in
    ``entrypoints``.

    This method must be called before the container is started.

    **Usage**

    The following service definition has two entrypoints for "method"::

        class Service(object):

            @rpc
            def foo(self, arg):
                pass

            @rpc
            @event_handler('srcservice', 'event_one')
            def bar(self, arg)
                pass

        container = container_factory(Service, config)

    To disable the entrypoints other than on "foo"::

        restrict_entrypoints(container, "foo")

    To maintain both the rpc and the event_handler entrypoints on "bar"::

        restrict_entrypoints(container, "bar")

    Note that it is not possible to identify entrypoints individually.
    """
    if container.started:
        raise RuntimeError('You must restrict entrypoints before the '
                           'container is started.')

    entrypoint_deps = list(container.entrypoints)
    entrypoint_names = {dep.name for dep in entrypoint_deps}

    missing = set(entrypoints) - entrypoint_names
    if missing:
        raise DependencyNotFound("Entrypoint(s) '{}' not found on {}.".format(
            missing, container))

    for dependency in entrypoint_deps:
        if dependency.name not in entrypoints:
            container.dependencies.remove(dependency)


class DummyEntrypoint(EntrypointProvider):
    pass


@entrypoint
def dummy():
    return DependencyFactory(DummyEntrypoint)



""" Barebones conftest.py
"""
# Nameko relies on eventlet
# You should monkey patch the standard library as early as possible to avoid
# importing anything before the patch is applied.
# See http://eventlet.net/doc/patching.html#monkeypatching-the-standard-library
import eventlet
eventlet.monkey_patch()

from urlparse import urlparse

from pyrabbit.api import Client
import pytest


def pytest_addoption(parser):
    parser.addoption(
        "--amqp-uri", action="store", dest='AMQP_URI',
        default='amqp://guest:guest@localhost:5672/nameko',
        help=("The AMQP-URI to connect to rabbit with."))

    parser.addoption(
        "--rabbit-ctl-uri", action="store", dest='RABBIT_CTL_URI',
        default='http://guest:guest@localhost:15672',
        help=("The URI for rabbit's management API."))


def pytest_configure(config):
    # monkey patch an encoding attribute onto GreenPipe to
    # satisfy a pytest assertion
    import py
    from eventlet.greenio import GreenPipe
    GreenPipe.encoding = py.std.sys.stdout.encoding


@pytest.fixture(autouse=True)
def reset_kombu_pools(request):
    from kombu.pools import reset
    reset()


@pytest.fixture(scope='session')
def rabbit_manager(request):
    config = request.config

    rabbit_ctl_uri = urlparse(config.getoption('RABBIT_CTL_URI'))
    host_port = '{0.hostname}:{0.port}'.format(rabbit_ctl_uri)

    rabbit = Client(
        host_port, rabbit_ctl_uri.username, rabbit_ctl_uri.password)

    return rabbit


@pytest.yield_fixture
def rabbit_config(request, rabbit_manager):
    amqp_uri = request.config.getoption('AMQP_URI')

    conf = {'AMQP_URI': amqp_uri}

    uri = urlparse(amqp_uri)
    vhost = uri.path[1:].replace('/', '%2F')
    username = uri.username

    conf['vhost'] = vhost
    conf['username'] = username

    def del_vhost():
        try:
            rabbit_manager.delete_vhost(vhost)
        except:
            pass

    del_vhost()
    rabbit_manager.create_vhost(vhost)
    rabbit_manager.set_vhost_permissions(vhost, username, '.*', '.*', '.*')

    yield conf

    del_vhost()



import sys

import eventlet
from eventlet import spawn, sleep, Timeout
from eventlet.event import Event
import greenlet
from mock import patch, call, ANY, Mock
import pytest

from nameko.containers import ServiceContainer, WorkerContext
from nameko.constants import MAX_WORKERS_CONFIG_KEY
from nameko.dependencies import(
    InjectionProvider, EntrypointProvider, entrypoint, injection,
    DependencyFactory)
from nameko.testing.utils import AnyInstanceOf


class CallCollectorMixin(object):
    call_counter = 0

    def __init__(self):
        self.instances.add(self)
        self._reset_calls()

    def _reset_calls(self):
        self.calls = []
        self.call_ids = []

    def _log_call(self, data):
        CallCollectorMixin.call_counter += 1
        self.calls.append(data)
        self.call_ids.append(CallCollectorMixin.call_counter)

    def prepare(self):
        self._log_call(('prepare'))
        super(CallCollectorMixin, self).prepare()

    def start(self):
        self._log_call(('start'))
        super(CallCollectorMixin, self).start()

    def stop(self):
        self._log_call(('stop'))
        super(CallCollectorMixin, self).stop()

    def kill(self, exc=None):
        self._log_call(('kill'))
        super(CallCollectorMixin, self).stop()

    def worker_setup(self, worker_ctx):
        self._log_call(('setup', worker_ctx))
        super(CallCollectorMixin, self).worker_setup(worker_ctx)

    def worker_result(self, worker_ctx, result=None, exc_info=None):
        self._log_call(('result', worker_ctx, (result, exc_info)))
        super(CallCollectorMixin, self).worker_result(
            worker_ctx, result, exc_info)

    def worker_teardown(self, worker_ctx):
        self._log_call(('teardown', worker_ctx))
        super(CallCollectorMixin, self).worker_teardown(worker_ctx)


class CallCollectingEntrypointProvider(
        CallCollectorMixin, EntrypointProvider):
    instances = set()


class CallCollectingInjectionProvider(
        CallCollectorMixin, InjectionProvider):
    instances = set()

    def acquire_injection(self, worker_ctx):
        self._log_call(('acquire', worker_ctx))
        return 'spam-attr'


@entrypoint
def foobar():
    return DependencyFactory(CallCollectingEntrypointProvider)


@injection
def call_collector():
    return DependencyFactory(CallCollectingInjectionProvider)


egg_error = Exception('broken')


class Service(object):
    name = 'test-service'

    spam = call_collector()

    @foobar
    def ham(self):
        return 'ham'

    @foobar
    def egg(self):
        raise egg_error

    @foobar
    def wait(self):
        while True:
            sleep()


@pytest.fixture
def container():
    container = ServiceContainer(service_cls=Service,
                                 worker_ctx_cls=WorkerContext,
                                 config={})
    for dep in container.dependencies:
        dep._reset_calls()

    CallCollectorMixin.call_counter = 0
    return container


@pytest.yield_fixture
def logger():
    with patch('nameko.containers._log', autospec=True) as patched:
        yield patched


def test_collects_dependencies(container):
    assert len(container.dependencies) == 4
    assert container.dependencies == (
        CallCollectingEntrypointProvider.instances |
        CallCollectingInjectionProvider.instances)


def test_starts_dependencies(container):

    for dep in container.dependencies:
        assert dep.calls == []

    container.start()

    for dep in container.dependencies:
        assert dep.calls == [
            ('prepare'),
            ('start')
        ]


def test_stops_dependencies(container):

    container.stop()
    for dep in container.dependencies:
        assert dep.calls == [
            ('stop')
        ]


def test_stops_entrypoints_before_injections(container):
    container.stop()

    dependencies = container.dependencies
    spam_dep = next(iter(dependencies.injections))

    for dec_dep in dependencies.entrypoints:
        assert dec_dep.call_ids[0] < spam_dep.call_ids[0]


def test_worker_life_cycle(container):
    dependencies = container.dependencies

    (spam_dep,) = [dep for dep in dependencies if dep.name == 'spam']
    (ham_dep,) = [dep for dep in dependencies if dep.name == 'ham']
    (egg_dep,) = [dep for dep in dependencies if dep.name == 'egg']

    handle_result = Mock()
    handle_result.side_effect = (
        lambda worker_ctx, res, exc_info: (res, exc_info))

    ham_worker_ctx = container.spawn_worker(
        ham_dep, [], {}, handle_result=handle_result)
    container._worker_pool.waitall()

    egg_worker_ctx = container.spawn_worker(
        egg_dep, [], {}, handle_result=handle_result)
    container._worker_pool.waitall()

    assert spam_dep.calls == [
        ('acquire', ham_worker_ctx),
        ('setup', ham_worker_ctx),
        ('result', ham_worker_ctx, ('ham', None)),
        ('teardown', ham_worker_ctx),
        ('acquire', egg_worker_ctx),
        ('setup', egg_worker_ctx),
        ('result', egg_worker_ctx, (None, (Exception, egg_error, ANY))),
        ('teardown', egg_worker_ctx),
    ]

    assert ham_dep.calls == [
        ('setup', ham_worker_ctx),
        ('teardown', ham_worker_ctx),
        ('setup', egg_worker_ctx),
        ('teardown', egg_worker_ctx),
    ]

    assert egg_dep.calls == [
        ('setup', ham_worker_ctx),
        ('teardown', ham_worker_ctx),
        ('setup', egg_worker_ctx),
        ('teardown', egg_worker_ctx),
    ]

    assert handle_result.call_args_list == [
        call(ham_worker_ctx, "ham", None),
        call(egg_worker_ctx, None, (Exception, egg_error, ANY))
    ]


def test_wait_waits_for_container_stopped(container):
    gt = spawn(container.wait)

    with Timeout(1):
        assert not gt.dead
        container.stop()
        sleep(0.01)
        assert gt.dead


def test_container_doesnt_exhaust_max_workers(container):
    spam_called = Event()
    spam_continue = Event()

    class Service(object):
        name = 'max-workers'

        @foobar
        def spam(self, a):
            spam_called.send(a)
            spam_continue.wait()

    container = ServiceContainer(service_cls=Service,
                                 worker_ctx_cls=WorkerContext,
                                 config={MAX_WORKERS_CONFIG_KEY: 1})

    dep = next(iter(container.dependencies))

    # start the first worker, which should wait for spam_continue
    container.spawn_worker(dep, ['ham'], {})

    # start the next worker in a speparate thread,
    # because it should block until the first one completed
    gt = spawn(container.spawn_worker, dep, ['eggs'], {})

    with Timeout(1):
        assert spam_called.wait() == 'ham'
        # if the container had spawned the second worker, we would see
        # an error indicating that spam_called was fired twice, and the
        # greenthread would now be dead.
        assert not gt.dead
        # reset the calls and allow the waiting worker to complete.
        spam_called.reset()
        spam_continue.send(None)
        # the second worker should now run and complete
        assert spam_called.wait() == 'eggs'
        assert gt.dead


def test_stop_already_stopped(container, logger):

    assert not container._died.ready()
    container.stop()
    assert container._died.ready()

    container.stop()
    assert logger.debug.call_args == call("already stopped %s", container)


def test_kill_already_stopped(container, logger):

    assert not container._died.ready()
    container.stop()
    assert container._died.ready()

    container.kill()
    assert logger.debug.call_args == call("already stopped %s", container)


def test_kill_container_with_active_threads(container):
    """ Start a thread that's not managed by dependencies. Ensure it is killed
    when the container is.
    """
    def sleep_forever():
        while True:
            sleep()

    container.spawn_managed_thread(sleep_forever)

    assert len(container._active_threads) == 1
    (worker_gt,) = container._active_threads

    container.kill()

    with Timeout(1):
        container._died.wait()

        with pytest.raises(greenlet.GreenletExit):
            worker_gt.wait()


def test_kill_container_with_protected_threads(container):
    """ Start a protected thread that's not managed by dependencies. Ensure it
    is killed when the container is.
    """
    def sleep_forever():
        while True:
            sleep()

    container.spawn_managed_thread(sleep_forever, protected=True)

    assert len(container._protected_threads) == 1
    (worker_gt,) = container._protected_threads

    container.kill()

    with Timeout(1):
        container._died.wait()

        with pytest.raises(greenlet.GreenletExit):
            worker_gt.wait()


def test_handle_killed_worker(container, logger):

    dep = next(iter(container.dependencies))
    container.spawn_worker(dep, ['sleep'], {})

    assert len(container._active_threads) == 1
    (worker_gt,) = container._active_threads

    worker_gt.kill()
    assert logger.warning.call_args == call("%s thread killed by container",
                                            container)

    assert not container._died.ready()  # container continues running


def test_spawned_thread_kills_container(container):
    def raise_error():
        raise Exception('foobar')

    container.start()
    container.spawn_managed_thread(raise_error)

    with pytest.raises(Exception) as exc_info:
        container.wait()

    assert exc_info.value.args == ('foobar',)


def test_spawned_thread_causes_container_to_kill_other_thread(container):
    killed_by_error_raised = Event()

    def raise_error():
        raise Exception('foobar')

    def wait_forever():
        try:
            Event().wait()
        except:
            killed_by_error_raised.send()
            raise

    container.start()

    container.spawn_managed_thread(wait_forever)
    container.spawn_managed_thread(raise_error)

    with Timeout(1):
        killed_by_error_raised.wait()


def test_container_only_killed_once(container):

    class Broken(Exception):
        pass

    exc = Broken('foobar')

    def raise_error():
        raise exc

    with patch.object(
            container, '_kill_active_threads', autospec=True) as kill_threads:

        with patch.object(container, 'kill', wraps=container.kill) as kill:
            # insert an eventlet yield into the kill process, otherwise
            # the container dies before the second exception gets raised
            kill.side_effect = lambda exc: sleep()

            container.start()

            # any exception in a managed thread will cause the container
            # to be killed. Two threads raising an exception will cause
            # the container's handle_thread_exit() method to kill the
            # container twice.
            container.spawn_managed_thread(raise_error)
            container.spawn_managed_thread(raise_error)

            # container should die with an exception
            with pytest.raises(Broken):
                container.wait()

            # container.kill should have been called twice
            assert kill.call_args_list == [
                call((Broken, exc, ANY)),
                call((Broken, exc, ANY))
            ]

            # only the first kill results in any cleanup
            assert kill_threads.call_count == 1


def test_container_stop_kills_remaining_managed_threads(container, logger):
    """ Verify any remaining managed threads are killed when a container
    is stopped.
    """
    def sleep_forever():
        while True:
            sleep()

    container.start()

    container.spawn_managed_thread(sleep_forever)
    container.spawn_managed_thread(sleep_forever, protected=True)

    container.stop()

    assert logger.warning.call_args_list == [
        call("killing %s active thread(s)", 1),
        call("%s thread killed by container", container),
        call("killing %s protected thread(s)", 1),
        call("%s thread killed by container", container),
    ]


def test_container_kill_kills_remaining_managed_threads(container, logger):
    """ Verify any remaining managed threads are killed when a container
    is killed.
    """
    def sleep_forever():
        while True:
            sleep()

    container.start()

    container.spawn_managed_thread(sleep_forever)
    container.spawn_managed_thread(sleep_forever, protected=True)

    container.kill()

    assert logger.warning.call_args_list == [
        call("killing %s active thread(s)", 1),
        call("%s thread killed by container", container),
        call("killing %s protected thread(s)", 1),
        call("%s thread killed by container", container),
    ]


def test_stop_during_kill(container, logger):
    """ Verify we handle the race condition when a runner tries to stop
    a container while it is being killed.
    """
    with patch.object(
            container, '_kill_active_threads', autospec=True) as kill_threads:

        # force eventlet yield during kill() so stop() will be scheduled
        kill_threads.side_effect = eventlet.sleep

        # manufacture an exc_info to kill with
        try:
            raise Exception('error')
        except:
            pass
        exc_info = sys.exc_info()

        eventlet.spawn(container.kill, exc_info)
        eventlet.spawn(container.stop)

        with pytest.raises(Exception):
            container.wait()
        assert logger.debug.call_args_list == [
            call("already being killed %s", container),
        ]


def test_container_entrypoint_property(container):
    entrypoints = container.entrypoints
    assert {dep.name for dep in entrypoints} == {'wait', 'ham', 'egg'}
    assert entrypoints == 3 * [AnyInstanceOf(CallCollectingEntrypointProvider)]


def test_container_injection_property(container):
    injections = container.injections
    assert {dep.name for dep in injections} == {'spam'}
    assert injections == [AnyInstanceOf(CallCollectingInjectionProvider)]


def test_container_catches_managed_thread_errors(container):

    class Broken(Exception):
        pass

    def raises():
        raise Broken('error')

    container.spawn_managed_thread(raises)

    with pytest.raises(Broken):
        container.wait()



import json
import sys

from mock import ANY
import pytest

from nameko.dependencies import (
    InjectionProvider, DependencyFactory, injection, entrypoint)
from nameko.exceptions import RemoteError
from nameko.rpc import RpcProvider
from nameko.standalone.rpc import RpcProxy
from nameko.testing.utils import wait_for_worker_idle


worker_result_called = []


@pytest.fixture(autouse=True)
def reset():
    del worker_result_called[:]


class CollectorInjection(InjectionProvider):
    """ InjectionProvider that collects worker results
    """
    def acquire_injection(self, worker_ctx):
        pass

    def worker_result(self, worker_ctx, res, exc_info):
        worker_result_called.append((res, exc_info))


@injection
def result_collector():
    return DependencyFactory(CollectorInjection)


class CustomRpcProvider(RpcProvider):
    """ RpcProvider subclass that verifies `result` can be serialized to json,
    and changes the `result` and `exc_info` accordingly.
    """
    def handle_result(self, message, worker_ctx, result, exc_info):
        try:
            json.dumps(result)
        except Exception:
            result = "something went wrong"
            exc_info = sys.exc_info()

        return super(CustomRpcProvider, self).handle_result(
            message, worker_ctx, result, exc_info)


@entrypoint
def custom_rpc():
    return DependencyFactory(CustomRpcProvider)


class ExampleService(object):

    collector = result_collector()

    @custom_rpc
    def echo(self, arg):
        return arg

    @custom_rpc
    def unserializable(self):
        return object()


def test_handle_result(container_factory, rabbit_manager, rabbit_config):
    """ Verify that `handle_result` can modify the return values of the worker,
    such that other dependencies see the updated values.
    """
    container = container_factory(ExampleService, rabbit_config)
    container.start()

    with RpcProxy('exampleservice', rabbit_config) as proxy:

        assert proxy.echo("hello") == "hello"
        with pytest.raises(RemoteError) as exc:
            proxy.unserializable()
        assert "is not JSON serializable" in exc.value.message

    wait_for_worker_idle(container)

    # verify CollectorInjection sees values returned from `handle_result`
    assert worker_result_called == [
        ("hello", None),
        ("something went wrong", (TypeError, ANY, ANY)),
    ]



"""
Provides a high level interface to the core messaging module.

Events are special messages, which can be emitted by one service
and handled by other listening services.

To emit an event, a service must define an :class:`Event` class with a unique
type and dispatch an instance of it using an injection acquired from an
instance of :class:`EventDispatcher`.

Events are dispatched asynchronously. It is only guaranteed that the event has
been dispatched, not that it was received or handled by a listener.

To listen to an event, a service must declare a handler using the
:func:`handle_event` entrypoint, providing the target service and an event type
filter.

Example::

    class MyEvent(Event):
        type = "myevent"

    @handle_event("foo_service", "myevent")
    def bar(evt):
        pass

"""
from __future__ import absolute_import
from logging import getLogger
import uuid

from kombu import Exchange, Queue

from nameko.constants import DEFAULT_RETRY_POLICY
from nameko.messaging import PublishProvider, PERSISTENT, ConsumeProvider
from nameko.dependencies import entrypoint, injection, DependencyFactory


SERVICE_POOL = "service_pool"
SINGLETON = "singleton"
BROADCAST = "broadcast"

_log = getLogger(__name__)


def get_event_exchange(service_name):
    """ Get an exchange for ``service_name`` events.
    """
    exchange_name = "{}.events".format(service_name)
    exchange = Exchange(
        exchange_name, type='topic', durable=True, auto_delete=True,
        delivery_mode=PERSISTENT)

    return exchange


class EventTypeMissing(Exception):
    """ Raised when an Event subclasses are defined without and event-type.
    """
    def __init__(self, name):
        msg = ("Event subclass '{}' cannot be created without "
               "a 'type' attribute.").format(name)

        super(EventTypeMissing, self).__init__(msg)


class EventTypeTooLong(Exception):
    """ Raised when event types are defined and longer than 255 bytes.
    """
    def __init__(self, event_type):
        msg = 'Event type "{}" too long. Should be < 255 bytes.'.format(
            event_type)
        super(EventTypeTooLong, self).__init__(msg)


class EventHandlerConfigurationError(Exception):
    """ Raised when an event handler is misconfigured.
    """


class EventMeta(type):
    """ Ensures every Event subclass has it's own event-type defined,
    and that the type is less than 255 bytes in size.

    This is a limitation imposed by AMQP topic exchanges.
    """

    def __new__(mcs, name, bases, dct):
        try:
            event_type = dct['type']
        except KeyError:
            raise EventTypeMissing(name)
        else:
            if len(event_type) > 255:
                raise EventTypeTooLong(event_type)

        return super(EventMeta, mcs).__new__(mcs, name, bases, dct)


class Event(object):
    """ The base class for all events to be dispatched by an `EventDispatcher`.
    """
    __metaclass__ = EventMeta

    type = 'Event'
    """ The type of the event.

    Events can be name-spaced using the type property:
    e.g.: ``type = 'spam.ham.eggs'``

    See amqp routing keys for `topic` exchanges for more info.
    """

    def __init__(self, data):
        self.data = data


class EventDispatcher(PublishProvider):
    """ Provides an event dispatcher method via dependency injection.

    Events emitted will be dispatched via the service's events exchange,
    which automatically gets declared by the event dispatcher
    as a topic exchange.
    The name for the exchange will be `{service-name}.events`.

    Events, emitted via the dispatcher, will be serialized and published
    to the events exchange. The event's type attribute is used as the
    routing key, which can be used for filtering on the listener's side.

    The dispatcher will return as soon as the event message has been published.
    There is no guarantee that any service will receive the event, only
    that the event has been successfully dispatched.

    Example::

        class MyEvent(Event):
            type = 'spam.ham'


        class Spammer(object):
            dispatch_spam = EventDispatcher()

            def emit_spam(self):
                evt = MyEvent('ham and eggs')
                self.dispatch_spam(evt)

    """
    def prepare(self):
        service_name = self.container.service_name
        self.exchange = get_event_exchange(service_name)
        super(EventDispatcher, self).prepare()

    def acquire_injection(self, worker_ctx):
        """ Inject a dispatch method onto the service instance
        """
        def dispatch(evt, **kwargs):
            exchange = self.exchange

            retry = kwargs.pop('retry', True)
            retry_policy = kwargs.pop('retry_policy', DEFAULT_RETRY_POLICY)

            msg = evt.data
            routing_key = evt.type

            with self.get_producer() as producer:

                headers = self.get_message_headers(worker_ctx)
                producer.publish(msg, exchange=exchange, headers=headers,
                                 routing_key=routing_key, retry=retry,
                                 retry_policy=retry_policy, **kwargs)

        return dispatch


@injection
def event_dispatcher():
    return DependencyFactory(EventDispatcher)


class EventHandler(ConsumeProvider):

    def __init__(self, service_name, event_type, handler_type,
                 reliable_delivery, requeue_on_error):

        self.service_name = service_name
        self.event_type = event_type
        self.handler_type = handler_type
        self.reliable_delivery = reliable_delivery

        super(EventHandler, self).__init__(
            queue=None, requeue_on_error=requeue_on_error)

    def prepare(self):
        _log.debug('starting handler for %s', self.container)

        # handler_type determines queue name
        service_name = self.container.service_name
        if self.handler_type is SERVICE_POOL:
            queue_name = "evt-{}-{}--{}.{}".format(self.service_name,
                                                   self.event_type,
                                                   service_name,
                                                   self.name)
        elif self.handler_type is SINGLETON:
            queue_name = "evt-{}-{}".format(self.service_name,
                                            self.event_type)
        elif self.handler_type is BROADCAST:
            queue_name = "evt-{}-{}--{}.{}-{}".format(self.service_name,
                                                      self.event_type,
                                                      service_name,
                                                      self.name,
                                                      uuid.uuid4().hex)

        exchange = get_event_exchange(self.service_name)

        # auto-delete queues if events are not reliably delivered
        auto_delete = not self.reliable_delivery
        self.queue = Queue(
            queue_name, exchange=exchange, routing_key=self.event_type,
            durable=True, auto_delete=auto_delete)

        super(EventHandler, self).prepare()


@entrypoint
def event_handler(service_name, event_type, handler_type=SERVICE_POOL,
                  reliable_delivery=True, requeue_on_error=False,
                  event_handler_cls=EventHandler):
    r"""
    Decorate a method as a handler of ``event_type`` events on the service
    called ``service_name``. ``event_type`` must be either a subclass of
    :class:`~.Event` with a class attribute ``type`` or a string matching the
    value of this attribute.
    ``handler_type`` determines the behaviour of the handler:

        - ``events.SERVICE_POOL``:

            Event handlers will be pooled by service type and handler-method
            and one from each pool will receive the event. ::

                           .-[queue]- (service X handler-method-1)
                          /
                exchange o --[queue]- (service X handler-method-2)
                          \
                           \          (service Y(instance 1) hanlder-method)
                            \       /
                             [queue]
                                    \
                                      (service Y(instance 2) handler-method)


        - ``events.SINGLETON``:

            Events will be received by only one registered handler.
            If requeued on error, they may be given to a different
            handler. ::

                                       (service X handler-method)
                                     /
                exchange o -- [queue]
                                     \
                                       (service Y handler-method)

        - ``events.BROADCAST``:
            Events will be received by every handler. This  will broadcast
            to every service instance, not just every service type
            - use wisely! ::

                            [queue]- (service X(instance 1) handler-method)
                          /
                exchange o - [queue]- (service X(instance 2) handler-method)
                          \
                            [queue]- (service Y handler-method)

    If ``requeue_on_error``, handlers will return the event to the queue if an
    error occurs while handling it. Defaults to False.

    If ``reliable_delivery``, events will be kept in the queue until there is
    a handler to consume them. Defaults to ``True``.

    ``event_handler_cls`` may be specified to use a different EventHandler
        (sub)class for custom behaviour.

    Raises an ``EventHandlerConfigurationError`` if the ``handler_type``
    is set to ``BROADCAST`` and ``reliable_delivery`` is set to ``True``.
    """

    if reliable_delivery and handler_type is BROADCAST:
        raise EventHandlerConfigurationError(
            "Broadcast event handlers cannot be configured with reliable "
            "delivery.")

    if isinstance(event_type, type) and issubclass(event_type, Event):
        event_type = event_type.type
    elif not isinstance(event_type, basestring):
        raise TypeError(
            'event_type must be either a nameko.events.Event subclass or a '
            'string a string matching the Event.type value. '
            'Got {}'.format(type(event_type).__name__))

    return DependencyFactory(event_handler_cls, service_name, event_type,
                             handler_type, reliable_delivery, requeue_on_error)



"""
Provides classes and method to deal with dependency injection.
"""
from __future__ import absolute_import

from abc import ABCMeta, abstractmethod
from functools import wraps, partial
import inspect
from itertools import chain
import types
from weakref import WeakSet, WeakKeyDictionary

from eventlet.event import Event
from nameko.utils import SpawningSet

from logging import getLogger
_log = getLogger(__name__)

ENTRYPOINT_PROVIDERS_ATTR = 'nameko_entrypoints'


# constants for dependency sharing
CONTAINER_SHARED = object()
# PROCESS_SHARED also serves as weakref-able sentinel obj
PROCESS_SHARED = type('process', (), {})()


class NotInitializedError(Exception):
    pass


class DependencyTypeError(TypeError):
    pass


class DependencyProvider(object):

    def prepare(self):
        """ Called when the service container starts.

        DependencyProviders should do any required initialisation here.
        """

    def start(self):
        """ Called when the service container has successfully started.

        This is only called after all other DependencyProviders have
        successfully initialised. If the DependencyProvider listens to
        external events, they may now start acting upon them.
        """

    def stop(self):
        """ Called when the service container begins to shut down.

        DependencyProviders should do any graceful shutdown here.
        """

    def kill(self):
        """ Called to stop this dependency without grace.

        DependencyProviders should urgently shut down here. This means
        stopping as soon as possible with ommiting important cleanup.
        This may be distinct from ``stop()`` for certain dependencies.

        For example, :class:`~messaging.QueueConsumer` tracks messages being
        processed and pending message acks. Its ``kill`` implementation
        discards these and disconnects from rabbit as soon as possible.
        """

    def worker_setup(self, worker_ctx):
        """ Called before a service worker executes a task. This method is
        called for all DependencyProviders, not just the one that triggered
        the worker spawn.

        DependencyProviders should do any pre-processing here, raising
        exceptions in the event of failure.

        Example: ...

        Args:
            - worker_ctx: see
                ``nameko.containers.ServiceContainer.spawn_worker``
        """

    def worker_teardown(self, worker_ctx):
        """ Called after a service worker has executed a task. This method is
        called for all DependencyProviders, not just the one that triggered
        the worker spawn.

        DependencyProviders should do any post-processing here, raising
        exceptions in the event of failure.

        Example: a database session provider may commit the session

        Args:
            - worker_ctx: see
                ``nameko.containers.ServiceContainer.spawn_worker``
        """

    def bind(self, name, container):
        """ Bind this DependencyProvider instance to ``container`` using the
        given ``name`` to identify the resource on the hosted service.

        Called during ServiceContainer initialisation. The DependencyProvider
        instance is created and then bound to the ServiceContainer instance
        controlling its lifecyle.
        """
        self.name = name
        self.container = container

    @property
    def nested_dependencies(self):
        """ Recusively yield nested dependencies of DependencyProvider.

        For example, with an instance of `:class:~nameko.rpc.RpcProvider`
        called ``rpc``::

            >>> deps = list(rpc.nested_dependencies)
            >>> deps
            [<nameko.rpc.RpcConsumer object at 0x10d125690>,
             <nameko.messaging.QueueConsumer object at 0x10d5b8f50>]
            >>>
        """
        for _, attr in inspect.getmembers(self):
            if isinstance(attr, DependencyProvider):
                yield attr
                for nested_dep in attr.nested_dependencies:
                    yield nested_dep

    def __str__(self):
        try:
            return '<{} [{}.{}] at 0x{:x}>'.format(
                type(self).__name__,
                self.container.service_name, self.name,
                id(self))
        except:
            return '<{} [unbound] at 0x{:x}>'.format(
                type(self).__name__, id(self))


class EntrypointProvider(DependencyProvider):
    pass


class InjectionProvider(DependencyProvider):
    __metaclass__ = ABCMeta

    @abstractmethod
    def acquire_injection(self, worker_ctx):
        """ A subclass must return the instance that should be injected
        into the worker instance of the service by the container.
        """

    def worker_result(self, worker_ctx, result=None, exc_info=None):
        """ Called with the result of a service worker execution.

        InjectionProvider that need to process the result should do it here.
        This method is called for all InjectionProviders on completion of any
        worker.

        Example: a database session provider may flush the transaction

        Args:
            - worker_ctx: see
                ``nameko.containers.ServiceContainer.spawn_worker``
        """

    def inject(self, worker_ctx):
        injection = self.acquire_injection(worker_ctx)

        injection_name = self.name
        service = worker_ctx.service
        setattr(service, injection_name, injection)

    def release(self, worker_ctx):

        service = worker_ctx.service
        injection_name = self.name
        delattr(service, injection_name)


class ProviderCollector(object):
    def __init__(self, *args, **kwargs):
        self._providers = set()
        self._providers_registered = False
        self._last_provider_unregistered = Event()
        super(ProviderCollector, self).__init__(*args, **kwargs)

    def register_provider(self, provider):
        self._providers_registered = True
        _log.debug('registering provider %s for %s', provider, self)
        self._providers.add(provider)

    def unregister_provider(self, provider):
        providers = self._providers
        if provider not in self._providers:
            return

        _log.debug('unregistering provider %s for %s', provider, self)

        providers.remove(provider)
        if len(providers) == 0:
            _log.debug('last provider unregistered for %s', self)
            self._last_provider_unregistered.send()

    def wait_for_providers(self):
        """ Wait for any providers registered with the collector to have
        unregistered.

        Returns immediately if no providers were ever registered.
        """
        if self._providers_registered:
            _log.debug('waiting for providers to unregister %s', self)
            self._last_provider_unregistered.wait()
            _log.debug('all providers unregistered %s', self)

    def stop(self):
        """ Default `:meth:DependencyProvider.stop()` implementation for
        subclasses using `ProviderCollector` as a mixin.
        """
        self.wait_for_providers()


class DependencySet(SpawningSet):

    @property
    def injections(self):
        """ A ``SpawningSet`` of just the ``InjectionProvider`` instances in
        this set.
        """
        return SpawningSet(item for item in self
                           if is_injection_provider(item))

    @property
    def entrypoints(self):
        """ A ``SpawningSet`` of just the ``EntrypointProvider`` instances in
        this set.
        """
        return SpawningSet(item for item in self
                           if is_entrypoint_provider(item))

    @property
    def nested(self):
        """ A ``SpawningSet`` of any nested dependency instances in this set.
        """
        all_deps = self
        return all_deps - self.injections - self.entrypoints


registered_dependencies = WeakSet()
registered_injections = WeakSet()


def register_dependency(factory):
    registered_dependencies.add(factory)


def register_injection(factory):
    registered_injections.add(factory)


def register_entrypoint(fn, provider):
    descriptors = getattr(fn, ENTRYPOINT_PROVIDERS_ATTR, None)

    if descriptors is None:
        descriptors = set()
        setattr(fn, ENTRYPOINT_PROVIDERS_ATTR, descriptors)

    descriptors.add(provider)


shared_dependencies = WeakKeyDictionary()


class DependencyFactory(object):

    sharing_key = None

    def __init__(self, dep_cls, *init_args, **init_kwargs):
        self.dep_cls = dep_cls
        self.args = init_args
        self.kwargs = init_kwargs

        # keep a reference to every created instance
        self.instances = WeakSet()

    @property
    def key(self):
        return (self.dep_cls, str(self.args), str(self.kwargs))

    def create_and_bind_instance(self, name, container):
        """ Instantiate ``dep_cls`` and bind it to ``container``.

        See `:meth:~DependencyProvider.bind`.
        """
        sharing_key = self.sharing_key
        if sharing_key is not None:
            if sharing_key is CONTAINER_SHARED:
                sharing_key = container

            shared_dependencies.setdefault(sharing_key, {})
            instance = shared_dependencies[sharing_key].get(self.key)
            if instance is None:
                instance = self.dep_cls(*self.args, **self.kwargs)
                shared_dependencies[sharing_key][self.key] = instance
        else:
            instance = self.dep_cls(*self.args, **self.kwargs)
        instance.bind(name, container)

        for name, attr in inspect.getmembers(instance):
            if isinstance(attr, DependencyFactory):
                prov = attr.create_and_bind_instance(name, container)
                setattr(instance, name, prov)

        self.instances.add(instance)
        return instance


def entrypoint(decorator_func):
    """ Transform a callable into a decorator that can be used to declare
    entrypoints.

    The callable must return a DependencyFactory that creates the
    EntrypointProvider instance.

    The returned ``wrapper`` function has a ``provider_cls`` attribute that
    references the EntrypointProvider subclass returned by its factory. This
    helps separate the ``@entrypoint`` decorated methods from the class
    that implements the dependency - users only need to refer to the method
    and nameko can determine the implementing class.

    e.g::

        @entrypoint
        def http(bind_port=80):
            return DependencyFactory(HttpEntrypoint, bind_port)

        class Service(object):

            @http
            def foobar():
                pass

    """
    def registering_decorator(wrapper, fn, args, kwargs):
        """ Verify that ``decorator_func`` returns a DependencyFactory and
        register ``fn`` as an entrypoint.

        Saves a reference to the provider class of the factory onto
        the ``wrapper`` function that wraps ``decorator_func``.
        """
        factory = decorator_func(*args, **kwargs)
        if not isinstance(factory, DependencyFactory):
            raise DependencyTypeError('Arguments to `entrypoint` must return '
                                      'DependencyFactory instances')
        register_entrypoint(fn, factory)
        wrapper.provider_cls = factory.dep_cls

        return fn

    @wraps(decorator_func)
    def wrapper(*args, **kwargs):
        if len(args) == 1 and isinstance(args[0], types.FunctionType):
            # usage without arguments to the decorator:
            # @foobar
            # def spam():
            #     pass
            return registering_decorator(wrapper, args[0], tuple(), {})
        else:
            # usage with arguments to the decorator:
            # @foobar('shrub', ...)
            # def spam():
            #     pass
            return partial(registering_decorator,
                           wrapper, args=args, kwargs=kwargs)

    return wrapper


def injection(fn):
    """ Transform a callable into a function that can be used to create
    injections.

    The callable must return a DependencyFactory that creates the
    InjectionProvider instance.

    The returned ``wrapped`` function has a ``provider_cls`` attribute that
    references the InjectionProvider subclass returned by the factory. This
    helps separate the ``@injection`` decorated methods from the class
    that implements the dependency - users only need to refer to the method
    and nameko can determine the implementing class.

    e.g::

        @injection
        def database(*args, **kwargs):
            return DependencyFactory(DatabaseProvider, *args, **kwargs)

        class Service(object):

            db = database()
    """
    @wraps(fn)
    def wrapped(*args, **kwargs):
        factory = fn(*args, **kwargs)
        if not isinstance(factory, DependencyFactory):
            raise DependencyTypeError('Arguments to `injection` must return '
                                      'DependencyFactory instances')
        register_injection(factory)
        wrapped.provider_cls = factory.dep_cls
        return factory
    return wrapped


def dependency(fn):
    @wraps(fn)
    def wrapped(*args, **kwargs):
        sharing_key = kwargs.pop('shared', None)

        factory = fn(*args, **kwargs)
        if not isinstance(factory, DependencyFactory):
            raise DependencyTypeError('Arguments to `dependency` must return '
                                      'DependencyFactory instances')
        factory.sharing_key = sharing_key
        register_dependency(factory)
        return factory
    return wrapped


def is_injection_provider(obj):
    return isinstance(obj, InjectionProvider)


def is_entrypoint_provider(obj):
    return isinstance(obj, EntrypointProvider)


def prepare_injection_providers(container, include_dependencies=False):
    service_cls = container.service_cls
    for name, attr in inspect.getmembers(service_cls):
        if attr in registered_injections:
            factory = attr
            provider = factory.create_and_bind_instance(name, container)
            yield provider
            if include_dependencies:
                for dependency in provider.nested_dependencies:
                    yield dependency


def prepare_entrypoint_providers(container, include_dependencies=False):
    service_cls = container.service_cls
    for name, attr in inspect.getmembers(service_cls, inspect.ismethod):
        factories = getattr(attr, ENTRYPOINT_PROVIDERS_ATTR, [])
        for factory in factories:
            provider = factory.create_and_bind_instance(name, container)
            yield provider
            if include_dependencies:
                for dependency in provider.nested_dependencies:
                    yield dependency


def prepare_dependencies(container):
    return chain(
        prepare_injection_providers(container, include_dependencies=True),
        prepare_entrypoint_providers(container, include_dependencies=True))



import uuid

from mock import Mock, call
import pytest

from nameko.events import event_handler, Event
from nameko.exceptions import RemoteError
from nameko.rpc import rpc
from nameko.standalone.rpc import RpcProxy


class ExampleEvent(Event):
    type = "example"


entrypoint_called = Mock()


class Service(object):

    @rpc
    def echo(self, arg):
        entrypoint_called(arg)
        return arg

    @rpc
    def broken(self):
        return uuid.uuid4()  # does not serialize

    @event_handler('service', ExampleEvent.type)
    def event(self, evt_data):
        entrypoint_called(evt_data)


def test_rpc_serializationx(container_factory, rabbit_config):

    container = container_factory(Service, rabbit_config)
    container.start()

    data = {
        "hello": ("world",),
        123: 456,
        'abc': [7, 8, 9],
        'foobar': 1.5,
    }
    expected = {
        "hello": ["world", ],
        '123': 456,
        'abc': [7, 8, 9],
        'foobar': 1.5,
    }

    with RpcProxy('service', rabbit_config) as proxy:
        assert proxy.echo(data) == expected
        assert entrypoint_called.call_args == call(expected)


def test_rpc_result_serialization_error(container_factory, rabbit_config):

    container = container_factory(Service, rabbit_config)
    container.start()

    with RpcProxy('service', rabbit_config) as proxy:
        with pytest.raises(RemoteError) as exc:
            proxy.broken()
        assert exc.value.exc_type == "UnserializableValueError"

        assert proxy.echo('foo') == "foo"  # subsequent calls ok


def test_rpc_proxy_serialization_error(container_factory, rabbit_config):

    container = container_factory(Service, rabbit_config)
    container.start()

    with RpcProxy('service', rabbit_config) as proxy:
        with pytest.raises(TypeError):
            proxy.echo(uuid.uuid4())

        assert proxy.echo('foo') == "foo"  # subsequent calls ok



import logging
from path import path
from .method_extractor import MethodExtractor
from .rst_render import RstPagePrinter


log = logging.getLogger(__name__)


class ServiceDocProcessor(object):
    def __init__(self, output, service_loader_function, event_loader_function):
        self.output = path(output)
        self.service_loader_function = service_loader_function
        self.event_loader_function = event_loader_function

    def write_docs(self):
        if not self.output.exists():
            self.output.mkdir_p()
        elif self.output.files():
            raise ValueError('The provided output folder is not empty.')

        extractor = MethodExtractor(
            self.service_loader_function,
            self.event_loader_function,
        )
        collection = extractor.extract()

        printer = RstPagePrinter(
            self.output
        )
        with printer:
            log.debug(collection.services)
            collection.render(printer)



from mock import Mock, patch, MagicMock, call
import pytest
from nameko.nameko_doc import rst_render as rst


@pytest.fixture
def renderer():
    return rst.RstPagePrinter(Mock())


def test_render_see_also():
    res = rst.render_see_also_section(
        contents=[
            rst.render_definition_list(
                contents=[
                    rst.render_definition(
                        term='Foo',
                        description='Bar'
                    ),
                    rst.render_definition(
                        term='Service Class',
                        description=rst.render_class_reference(
                            'nameko_doc.example_service.BarService',
                        )
                    )
                ]
            )
        ]
    ).strip()
    assert res == """
.. seealso::

    Foo
        Bar
    Service Class
        :class:`nameko_doc.example_service.BarService`""".strip()


def test_render_method():
    res = rst.render_include_method(
        path='foo.BarClass.method_name',
        no_index=True,
        extras=[
            rst.render_instruction(
                name='Listens to',
                content='a.thing',
            )
        ]
    ).strip()
    expected_lines = [
        '.. automethod:: foo.BarClass.method_name',
        '    :noindex:',
        '    ',
        '    :Listens to: a.thing',
    ]
    expected = '\n'.join(expected_lines)
    assert res == expected


def test_render_class():
    res = rst.render_include_class(
        path='foo.bar.events.EventClass',
        no_index=True,
        extras=[
            rst.render_instruction(
                name='Handled by',
                content='some.thing',
            )
        ]
    ).strip()
    expected_lines = [
        '.. autoclass:: foo.bar.events.EventClass',
        '    :noindex:',
        '    ',
        '    :Handled by: some.thing',
    ]
    expected = '\n'.join(expected_lines)
    assert res == expected


def test_render_section():
    res = rst.render_section(
        contents=[
            rst.render_title('Hello', level=1, as_code=True),
            rst.render_section(
                contents=[
                    rst.render_title('Bye', level=2),
                    rst.render_include_module(
                        path='hello.world',
                    ),
                    'Some Content',
                ]
            )
        ]
    ).strip()
    expected = """
``Hello``
=========

Bye
---

.. automodule:: hello.world

Some Content""".strip()
    assert res == expected


def test_render_page():
    res = rst.render_page(
        name='foo',
        parts=[
            rst.render_title('Three', level=3),
            rst.render_include_method(
                path='foo.BarClass.baz'
            )
        ]
    )
    expected_content_lines = [
        'Three',
        '~~~~~',
        '',
        '.. automethod:: foo.BarClass.baz',
        '    ',
        '',
    ]
    expected_content = '\n'.join(expected_content_lines)

    expected = rst.RenderedPage(
        filename='foo.rst',
        content=expected_content,
        title='foo'
    )

    assert res == expected


@pytest.yield_fixture
def mock_write():
    with patch('nameko.nameko_doc.rst_render.open', create=True) as m:
        m.return_value = MagicMock(spec=file)
        yield m, m.return_value.__enter__.return_value


def test_render_multiple_pages(renderer, mock_write):
    renderer.output = 'folder'
    mock_open, handle = mock_write

    with renderer:
        renderer.add_page(
            rst.RenderedPage(
                filename='foo.rst',
                content='Hello, World',
                title='foo',
            )
        )
        renderer.add_page(
            rst.RenderedPage(
                filename='bar.rst',
                content='Goodbye, World',
                title='bar',
            )
        )

    assert mock_open.call_args_list == [
        call('folder/index.rst', 'w'),
        call('folder/foo.rst', 'w'),
        call('folder/bar.rst', 'w'),
    ]

    assert handle.write.call_args_list == [
        call('Services\n========\n\n.. toctree::\n    :maxdepth: 3\n\n'
             '    bar\n    foo'),
        call('Hello, World'),
        call('Goodbye, World'),
    ]



from collections import namedtuple
import rst_render as rst


TITLE_LEVEL_FOR_SUBSECTIONS = 2


class ServiceCollection(namedtuple('ServiceCollection', [
    'services',
])):
    def render(self, rst_page_printer):
        """ Render this service collection out """
        for service in self.services:
            page = service.render_page()
            rst_page_printer.add_page(page)


class ServiceDescription(namedtuple('ServiceDescription', [
    'name', 'module_path', 'class_name', 'sections',
])):
    def render_page(self):
        """ Render a service into a page """
        section_parts = [
            section.render_section(self, TITLE_LEVEL_FOR_SUBSECTIONS)
            for section in self.sections
        ]

        page = rst.render_page(
            name=self.name,
            parts=[
                rst.render_title(
                    text=self.name,
                    as_code=True,
                    level=1,
                ),
                rst.render_include_module(
                    path=self.module_path,
                )
            ] + section_parts
        )
        return page


class ReferenceSection(namedtuple('ReferenceSection', [
    'references',
])):
    def render_section(self, service_description, this_level=1):
        aside = rst.render_see_also_section(
            contents=[
                rst.render_definition_list(
                    contents=[
                        ref.render_reference()
                        for ref in self.references
                    ]
                )
            ]
        )
        return aside


class ClassReference(namedtuple('ClassReference', [
    'title', 'reference_path',
])):
    def render_reference(self):
        """ Render a reference to a class under a given category/title """
        return rst.render_definition(
            term=self.title,
            description=rst.render_class_reference(
                path=self.reference_path
            )
        )


class Section(namedtuple('Section', [
    'title', 'contents',
])):
    def render_section(self, service_description, this_level=1):
        sub_section_contents = [
            content.render_section(service_description, this_level + 1)
            for content in self.contents
        ]
        section = rst.render_section(
            contents=[
                rst.render_title(
                    text=self.title,
                    level=this_level,
                )
            ] + sub_section_contents
        )
        return section


class SingleMethod(namedtuple('SingleMethod', [
    'method_name', 'extras',
])):
    def render_section(self, service_description, this_level=1):
        method_path = '{}.{}.{}'.format(
            service_description.module_path,
            service_description.class_name,
            self.method_name
        )

        method_ref = rst.render_include_method(
            path=method_path,
            no_index=True,
            extras=[
                extra.render_extra() for extra in self.extras
            ]
        )

        return method_ref


class ExtraInstruction(namedtuple('ExtraInstruction', [
    'title', 'content'
])):
    def render_extra(self):
        return rst.render_instruction(self.title, self.content)


class SingleEvent(namedtuple('SingleEvent', [
    'event_path', 'extras'
])):
    def render_section(self, service_description, this_level=1):
        event_ref = rst.render_include_class(
            path=self.event_path,
            no_index=True,
            extras=[
                extra.render_extra() for extra in self.extras
            ]
        )

        return event_ref



from contextlib import contextmanager

from kombu.common import maybe_declare
from kombu.pools import producers, connections
from kombu import Connection

from nameko.constants import DEFAULT_RETRY_POLICY
from nameko.events import get_event_exchange
from nameko.messaging import AMQP_URI_CONFIG_KEY


@contextmanager
def event_dispatcher(container_service_name, nameko_config, **kwargs):
    """ Yield a function that dispatches events claiming to originate from
    a service called `container_service_name`.

    Enables services not hosted by nameko to dispatch events into a nameko
    cluster.
    """
    conn = Connection(nameko_config[AMQP_URI_CONFIG_KEY])
    exchange = get_event_exchange(container_service_name)

    retry = kwargs.pop('retry', True)
    retry_policy = kwargs.pop('retry_policy', DEFAULT_RETRY_POLICY)

    with connections[conn].acquire(block=True) as connection:
        maybe_declare(exchange, connection)

        with producers[conn].acquire(block=True) as producer:

            def dispatch(evt):
                msg = evt.data
                routing_key = evt.type
                producer.publish(
                    msg, exchange=exchange, routing_key=routing_key,
                    retry=retry, retry_policy=retry_policy, **kwargs)

            yield dispatch



"""
In a large integration test of highly connected services, it's preferable to
limit interactions to only those under test.

This file defines several toy services that interact to form a shop of the
famous ACME Corporation. The AcmeShopService relies on the StockService,
InvoiceService and PaymentService to fulfil its orders. They are not best
practice examples! They're minimal services provided for the test at the
bottom of the file.

``test_shop_integration`` is a  full integration test of the ACME shop
"checkout" flow. It demonstrates how to test the multiple ACME services in
combination with each other, including limiting service interactions by
replacing certain entrypoint and injection dependencies.
"""

from collections import defaultdict

import pytest

from nameko.dependencies import (
    InjectionProvider, injection, DependencyFactory)
from nameko.events import event_dispatcher, Event, event_handler
from nameko.exceptions import RemoteError
from nameko.rpc import rpc, rpc_proxy
from nameko.runners import ServiceRunner
from nameko.standalone.rpc import RpcProxy
from nameko.testing.services import replace_injections, restrict_entrypoints
from nameko.testing.utils import get_container
from nameko.timer import timer


class NotLoggedIn(Exception):
    pass


class ItemOutOfStock(Exception):
    pass


class ItemDoesNotExist(Exception):
    pass


class ShoppingBasket(InjectionProvider):
    """ A shopping basket tied to the current ``user_id``.
    """
    def __init__(self):
        self.baskets = defaultdict(list)

    def acquire_injection(self, worker_ctx):

        class Basket(object):
            def __init__(self, basket):
                self._basket = basket
                self.worker_ctx = worker_ctx

            def add(self, item):
                self._basket.append(item)

            def __iter__(self):
                for item in self._basket:
                    yield item

        try:
            user_id = worker_ctx.data['user_id']
        except KeyError:
            raise NotLoggedIn()
        return Basket(self.baskets[user_id])


@injection
def shopping_basket():
    """ A shopping basket tied to the current user.
    """
    return DependencyFactory(ShoppingBasket)


class ItemAddedToBasket(Event):
    """ Dispatched when an item is added to a shopping basket.
    """
    type = "item_added_to_basket"


class CheckoutComplete(Event):
    """ Dispatched when the checkout process completes
    """
    type = "checkout_complete"


class AcmeShopService(object):

    user_basket = shopping_basket()
    stock_service = rpc_proxy('stockservice')
    invoice_service = rpc_proxy('invoiceservice')
    payment_service = rpc_proxy('paymentservice')

    fire_event = event_dispatcher()

    @rpc
    def add_to_basket(self, item_code):
        """ Add item identified by ``item_code`` to the shopping basket.

        This is a toy example! Ignore the obvious race condition.
        """
        stock_level = self.stock_service.check_stock(item_code)
        if stock_level > 0:
            self.user_basket.add(item_code)
            self.fire_event(ItemAddedToBasket(item_code))
            return item_code

        raise ItemOutOfStock(item_code)

    @rpc
    def checkout(self):
        """ Take payment for all items in the shopping basket.
        """
        total_price = sum(self.stock_service.check_price(item)
                          for item in self.user_basket)

        # prepare invoice
        invoice = self.invoice_service.prepare_invoice(total_price)

        # take payment
        self.payment_service.take_payment(invoice)

        # fire checkout event if prepare_invoice and take_payment succeeded
        checkout_event = CheckoutComplete({
            'invoice': invoice,
            'items': list(self.user_basket)
        })
        self.fire_event(checkout_event)
        return total_price


class Warehouse(InjectionProvider):
    """ A database of items in the warehouse.

    This is a toy example! A dictionary is not a database.
    """
    def __init__(self):
        self.database = {
            'anvil': {
                'price': 100,
                'stock': 3
            },
            'dehydrated_boulders': {
                'price': 999,
                'stock': 12
            },
            'invisible_paint': {
                'price': 10,
                'stock': 30
            },
            'toothpicks': {
                'price': 1,
                'stock': 0
            }
        }

    def acquire_injection(self, worker_ctx):
        return self.database


@injection
def warehouse():
    """ A shopping basket tied to the current user.
    """
    return DependencyFactory(Warehouse)


class StockService(object):

    warehouse = warehouse()

    @rpc
    def check_price(self, item_code):
        """ Check the price of an item.
        """
        try:
            return self.warehouse[item_code]['price']
        except KeyError:
            raise ItemDoesNotExist(item_code)

    @rpc
    def check_stock(self, item_code):
        """ Check the stock level of an item.
        """
        try:
            return self.warehouse[item_code]['stock']
        except KeyError:
            raise ItemDoesNotExist(item_code)

    @rpc
    @timer(100)
    def monitor_stock(self):
        """ Periodic stock monitoring method. Can also be triggered manually
        over RPC.

        This is an expensive process that we don't want to exercise during
        integration testing...
        """
        raise NotImplemented()

    @event_handler('acmeshopservice', CheckoutComplete.type)
    def dispatch_items(self, event_data):
        """ Dispatch items from stock on successful checkouts.

        This is an expensive process that we don't want to exercise during
        integration testing...
        """
        raise NotImplemented()


class AddressBook(InjectionProvider):
    """ A database of user details, keyed on user_id.
    """
    def __init__(self):
        self.address_book = {
            'wile_e_coyote': {
                'username': 'wile_e_coyote',
                'fullname': 'Wile E Coyote',
                'address': '12 Long Road, High Cliffs, Utah',
            },
        }

    def acquire_injection(self, worker_ctx):
        def get_user_details():
            try:
                user_id = worker_ctx.data['user_id']
            except KeyError:
                raise NotLoggedIn()
            return self.address_book.get(user_id)
        return get_user_details


@injection
def address_book():
    """ Provides the address of the current user.
    """
    return DependencyFactory(AddressBook)


class InvoiceService(object):

    get_user_details = address_book()

    @rpc
    def prepare_invoice(self, amount):
        """ Prepare an invoice for ``amount`` for the current user.
        """
        address = self.get_user_details().get('address')
        fullname = self.get_user_details().get('fullname')
        username = self.get_user_details().get('username')

        msg = "Dear {}. Please pay ${} to ACME Corp.".format(fullname, amount)
        invoice = {
            'message': msg,
            'amount': amount,
            'customer': username,
            'address': address
        }
        return invoice


class PaymentService(object):

    @rpc
    def take_payment(self, invoice):
        """ Take payment from a customer according to ``invoice``.

        This is an expensive process that we don't want to exercise during
        integration testing...
        """
        raise NotImplemented()

# =============================================================================
# Begin test
# =============================================================================


@pytest.yield_fixture
def runner_factory(rabbit_config):

    all_runners = []

    def make_runner(*service_classes):
        runner = ServiceRunner(rabbit_config)
        for service_cls in service_classes:
            runner.add_service(service_cls)
        all_runners.append(runner)
        return runner

    yield make_runner

    for r in all_runners:
        try:
            r.stop()
        except:
            pass


@pytest.yield_fixture
def rpc_proxy_factory(rabbit_config):
    """ Factory fixture for standalone RPC proxies.

    Proxies are started automatically so they can be used without a ``with``
    statement. All created proxies are stopped at the end of the test, when
    this fixture closes.
    """
    all_proxies = []

    def make_proxy(service_name, **kwargs):
        proxy = RpcProxy(service_name, rabbit_config, **kwargs)
        all_proxies.append(proxy)
        return proxy.start()

    yield make_proxy

    for proxy in all_proxies:
        proxy.stop()


def test_shop_checkout_integration(runner_factory, rpc_proxy_factory):
    """ Simulate a checkout flow as an integration test.

    Requires instances of AcmeShopService, StockService and InvoiceService
    to be running. Explicitly replaces the rpc proxy to PaymentService so
    that service doesn't need to be hosted.

    Also replaces the event dispatcher injection on AcmeShopService and
    disables the timer entrypoint on StockService. Limiting the interactions
    of services in this way reduces the scope of the integration test and
    eliminates undesirable side-effects (e.g. processing events unnecessarily).
    """
    context_data = {'user_id': 'wile_e_coyote'}
    shop = rpc_proxy_factory('acmeshopservice', context_data=context_data)

    runner = runner_factory(AcmeShopService, StockService, InvoiceService)

    # replace ``event_dispatcher`` and ``payment_service``  injections on
    # AcmeShopService with Mock injections
    shop_container = get_container(runner, AcmeShopService)
    fire_event, payment_service = replace_injections(
        shop_container, "fire_event", "payment_service")

    # restrict entrypoints on StockService
    stock_container = get_container(runner, StockService)
    restrict_entrypoints(stock_container, "check_price", "check_stock")

    runner.start()

    # add some items to the basket
    assert shop.add_to_basket("anvil") == "anvil"
    assert shop.add_to_basket("invisible_paint") == "invisible_paint"

    # try to buy something that's out of stock
    with pytest.raises(RemoteError) as exc_info:
        shop.add_to_basket("toothpicks")
    assert exc_info.value.exc_type == "ItemOutOfStock"

    # provide a mock response from the payment service
    payment_service.take_payment.return_value = "Payment complete."

    # checkout
    res = shop.checkout()

    total_amount = 100 + 10
    assert res == total_amount

    # verify integration with mocked out payment service
    payment_service.take_payment.assert_called_once_with({
        'customer': "wile_e_coyote",
        'address': "12 Long Road, High Cliffs, Utah",
        'amount': total_amount,
        'message': "Dear Wile E Coyote. Please pay $110 to ACME Corp."
    })

    # verify events fired as expected
    assert fire_event.call_count == 3


if __name__ == "__main__":
    import sys
    pytest.main(sys.argv)



from __future__ import absolute_import

from sqlalchemy import create_engine
from sqlalchemy.orm import sessionmaker

from nameko.dependencies import InjectionProvider, injection, DependencyFactory

ORM_DB_URIS_KEY = 'ORM_DB_URIS'


class OrmSession(InjectionProvider):
    def __init__(self, declarative_base):
        self.declarative_base = declarative_base
        self.sessions = {}

    def acquire_injection(self, worker_ctx):
        service_name = self.container.service_name
        decl_base_name = self.declarative_base.__name__
        uri_key = '{}:{}'.format(service_name, decl_base_name)

        db_uris = worker_ctx.config[ORM_DB_URIS_KEY]
        db_uri = db_uris[uri_key].format({
            'service_name': service_name,
            'declarative_base_name': decl_base_name,
        })

        engine = create_engine(db_uri)
        Session = sessionmaker(bind=engine)
        session = Session()

        self.sessions[worker_ctx] = session
        return session

    def worker_teardown(self, worker_ctx):
        session = self.sessions.pop(worker_ctx)
        session.close()


@injection
def orm_session(declarative_base):
    return DependencyFactory(OrmSession, declarative_base)



import eventlet
from eventlet.event import Event
from mock import Mock, patch, call
from pyrabbit.http import HTTPError
import pytest

from nameko.constants import DEFAULT_MAX_WORKERS
from nameko.rpc import rpc
from nameko.testing.utils import (
    AnyInstanceOf, get_dependency, get_container, wait_for_call,
    reset_rabbit_vhost, get_rabbit_connections, wait_for_worker_idle,
    reset_rabbit_connections)


def test_any_instance_of():

    assert "" == AnyInstanceOf(str)
    assert 99 != AnyInstanceOf(str)

    class Foo(object):
        pass
    foo = Foo()

    assert foo == AnyInstanceOf(Foo)
    assert foo == AnyInstanceOf(object)
    assert foo != AnyInstanceOf(foo)

    assert repr(AnyInstanceOf(str)) == "<AnyInstanceOf-str>"
    assert repr(AnyInstanceOf(Foo)) == "<AnyInstanceOf-Foo>"

    assert AnyInstanceOf == AnyInstanceOf(type)
    assert str == AnyInstanceOf(type)
    assert int == AnyInstanceOf(type)
    assert type == AnyInstanceOf(type)


def test_wait_for_call():
    mock = Mock()

    def call_after(seconds):
        eventlet.sleep(seconds)
        mock.method()

    # should not raise
    eventlet.spawn(call_after, 0)
    with wait_for_call(1, mock.method):
        pass

    mock.reset_mock()

    with pytest.raises(eventlet.Timeout):
        eventlet.spawn(call_after, 1)
        with wait_for_call(0, mock.method):
            pass


def test_get_dependency(rabbit_config):

    from nameko.messaging import QueueConsumer
    from nameko.rpc import RpcProvider, RpcConsumer
    from nameko.containers import ServiceContainer, WorkerContext

    class Service(object):
        @rpc
        def foo(self):
            pass

        @rpc
        def bar(self):
            pass

    container = ServiceContainer(Service, WorkerContext, rabbit_config)

    rpc_consumer = get_dependency(container, RpcConsumer)
    queue_consumer = get_dependency(container, QueueConsumer)
    foo_rpc = get_dependency(container, RpcProvider, name="foo")
    bar_rpc = get_dependency(container, RpcProvider, name="bar")

    all_deps = container.dependencies
    assert all_deps == set([rpc_consumer, queue_consumer, foo_rpc, bar_rpc])


def test_get_container(runner_factory, rabbit_config):

    class ServiceX(object):
        name = "service_x"

    class ServiceY(object):
        name = "service_y"

    runner = runner_factory(rabbit_config, ServiceX, ServiceY)

    assert get_container(runner, ServiceX).service_cls is ServiceX
    assert get_container(runner, ServiceY).service_cls is ServiceY
    assert get_container(runner, object) is None


def test_reset_rabbit_vhost(rabbit_config, rabbit_manager):

    vhost = rabbit_config['vhost']
    username = rabbit_config['username']

    def get_active_vhosts():
        return [vhost_data['name'] for
                vhost_data in rabbit_manager.get_all_vhosts()]

    reset_rabbit_vhost(vhost, username, rabbit_manager)
    assert vhost in get_active_vhosts()

    rabbit_manager.delete_vhost(vhost)
    assert vhost not in get_active_vhosts()

    reset_rabbit_vhost(vhost, username, rabbit_manager)
    assert vhost in get_active_vhosts()


def test_reset_rabbit_vhost_errors():

    rabbit_manager = Mock()

    # 500 error
    error_500 = HTTPError({'reason': 'error'}, status=500)
    rabbit_manager.delete_vhost.side_effect = error_500

    with pytest.raises(HTTPError):
        reset_rabbit_vhost("vhost", "username", rabbit_manager)

    # 404 error
    error_404 = HTTPError({'reason': 'error'}, status=404)
    rabbit_manager.delete_vhost.side_effect = error_404

    # does not raise
    reset_rabbit_vhost("vhost", "username", rabbit_manager)
    assert rabbit_manager.create_vhost.call_args == call("vhost")
    assert rabbit_manager.set_vhost_permissions.call_args == call(
        "vhost", "username", '.*', '.*', '.*')


def test_get_rabbit_connections():

    vhost = "vhost_name"

    connections = [{
        'vhost': vhost,
        'key': 'value'
    }, {
        'vhost': 'unlikely_vhost_name',
        'key': 'value'
    }]

    rabbit_manager = Mock()

    rabbit_manager.get_connections.return_value = connections
    vhost_conns = [connections[0]]
    assert get_rabbit_connections(vhost, rabbit_manager) == vhost_conns

    rabbit_manager.get_connections.return_value = None
    assert get_rabbit_connections(vhost, rabbit_manager) == []


def test_reset_rabbit_connections():

    with patch('nameko.testing.utils.get_rabbit_connections') as connections:
        connections.return_value = [{
            'vhost': 'vhost',
            'name': 'connection_name'
        }]

        rabbit_manager = Mock()
        reset_rabbit_connections('vhost', rabbit_manager)

        assert rabbit_manager.delete_connection.call_args_list == [
            call("connection_name")]


def test_reset_rabbit_connection_errors():

    rabbit_manager = Mock()

    with patch('nameko.testing.utils.get_rabbit_connections') as connections:
        connections.return_value = [{
            'vhost': 'vhost_name',
            'name': 'connection_name'
        }]

        # 500 error
        error_500 = HTTPError({'reason': 'err'}, status=500)
        rabbit_manager.delete_connection.side_effect = error_500

        with pytest.raises(HTTPError):
            reset_rabbit_connections("vhost_name", rabbit_manager)

        # 404 error
        error_404 = HTTPError({'reason': 'err'}, status=404)
        rabbit_manager.delete_connection.side_effect = error_404

        # does not raise
        reset_rabbit_connections("vhost_name", rabbit_manager)


def test_wait_for_worker_idle(container_factory, rabbit_config):

    event = Event()

    class Service(object):

        @rpc
        def wait_for_event(self):
            event.wait()

    container = container_factory(Service, rabbit_config)
    container.start()

    max_workers = DEFAULT_MAX_WORKERS

    # verify nothing running
    assert container._worker_pool.free() == max_workers
    with eventlet.Timeout(1):
        wait_for_worker_idle(container)

    # spawn a worker
    wait_for_event = get_dependency(container, rpc.provider_cls)
    container.spawn_worker(wait_for_event, [], {})

    # verify that wait_for_worker_idle does not return while worker active
    assert container._worker_pool.free() == max_workers - 1
    gt = eventlet.spawn(wait_for_worker_idle, container)
    assert not gt.dead  # still waiting

    # verify that wait_for_worker_idle raises when it times out
    with pytest.raises(eventlet.Timeout):
        wait_for_worker_idle(container, timeout=0)

    # complete the worker, verify previous wait_for_worker_idle completes
    event.send()
    with eventlet.Timeout(1):
        gt.wait()
    assert container._worker_pool.free() == max_workers



from mock import Mock

from nameko.events import Event, event_handler
from nameko.standalone.events import event_dispatcher
from nameko.testing.utils import wait_for_call

handler_called = Mock()


class TestEvent(Event):
    type = "testevent"


class Service(object):
    name = 'destservice'

    @event_handler('srcservice', 'testevent')
    def handler(self, msg):
        handler_called(msg)


def test_dispatch(container_factory, rabbit_config):
    config = rabbit_config

    container = container_factory(Service, config)
    container.start()

    msg = "msg"

    with event_dispatcher('srcservice', config) as dispatch:
        dispatch(TestEvent(msg))

        with wait_for_call(1, handler_called):
            handler_called.assert_called_once_with(msg)



import uuid

import eventlet
from eventlet.event import Event
from mock import patch, Mock, call
import pytest

from nameko.containers import (
    ServiceContainer, WorkerContextBase, NAMEKO_CONTEXT_KEYS)
from nameko.dependencies import InjectionProvider, injection, DependencyFactory
from nameko.events import event_handler
from nameko.exceptions import (
    RemoteError, MethodNotFound, UnknownService, IncorrectSignature)
from nameko.messaging import QueueConsumer
from nameko.rpc import (
    rpc, rpc_proxy, RpcConsumer, RpcProvider, ReplyListener)
from nameko.standalone.rpc import RpcProxy
from nameko.testing.services import entrypoint_hook
from nameko.testing.utils import get_dependency, wait_for_call


class ExampleError(Exception):
    pass


hello = object()
translations = {
    'en': {hello: 'hello'},
    'fr': {hello: 'bonjour'},
}


class Translator(InjectionProvider):

    def acquire_injection(self, worker_ctx):
        def translate(value):
            lang = worker_ctx.data['language']
            return translations[lang][value]
        return translate


@injection
def translator():
    return DependencyFactory(Translator)


class CustomWorkerContext(WorkerContextBase):
    context_keys = NAMEKO_CONTEXT_KEYS + ('custom_header',)


class ExampleService(object):
    name = 'exampleservice'

    translate = translator()
    example_rpc = rpc_proxy('exampleservice')
    unknown_rpc = rpc_proxy('unknown_service')

    @rpc
    def task_a(self, *args, **kwargs):
        print "task_a", args, kwargs
        return "result_a"

    @rpc
    def task_b(self, *args, **kwargs):
        print "task_b", args, kwargs
        return "result_b"

    @rpc
    def broken(self):
        raise ExampleError("broken")

    @rpc
    def call_unknown(self):
        return self.unknown_rpc.any_method()

    @rpc
    def echo(self, *args, **kwargs):
        return args, kwargs

    @rpc
    def say_hello(self):
        return self.translate(hello)

    @event_handler('srcservice', 'eventtype')
    def async_task(self):
        pass


@pytest.yield_fixture
def get_rpc_exchange():
    with patch('nameko.rpc.get_rpc_exchange', autospec=True) as patched:
        yield patched


def test_rpc_consumer(get_rpc_exchange):

    container = Mock(spec=ServiceContainer)
    container.service_name = "exampleservice"
    container.service_cls = Mock(rpcmethod=lambda: None)

    exchange = Mock()
    get_rpc_exchange.return_value = exchange

    queue_consumer = Mock(spec=QueueConsumer)
    queue_consumer.bind("queue_consumer", container)

    consumer = RpcConsumer()
    consumer.queue_consumer = queue_consumer
    consumer.bind("rpc_consumer", container)

    provider = RpcProvider()
    provider.rpc_consumer = consumer
    provider.bind("rpcmethod", container)

    provider.prepare()
    consumer.prepare()
    queue_consumer.prepare()

    queue = consumer.queue
    assert queue.name == "rpc-exampleservice"
    assert queue.routing_key == "exampleservice.*"
    assert queue.exchange == exchange
    assert queue.durable

    queue_consumer.register_provider.assert_called_once_with(consumer)

    consumer.register_provider(provider)
    assert consumer._providers == set([provider])

    routing_key = "exampleservice.rpcmethod"
    assert consumer.get_provider_for_method(routing_key) == provider

    routing_key = "exampleservice.invalidmethod"
    with pytest.raises(MethodNotFound):
        consumer.get_provider_for_method(routing_key)

    consumer.unregister_provider(provider)
    assert consumer._providers == set()


def test_reply_listener(get_rpc_exchange):

    container = Mock(spec=ServiceContainer)
    container.service_name = "exampleservice"

    exchange = Mock()
    get_rpc_exchange.return_value = exchange

    queue_consumer = Mock(spec=QueueConsumer)
    queue_consumer.bind("queue_consumer", container)

    reply_listener = ReplyListener()
    reply_listener.queue_consumer = queue_consumer
    reply_listener.bind("reply_listener", container)

    forced_uuid = uuid.uuid4().hex

    with patch('nameko.rpc.uuid', autospec=True) as patched_uuid:
        patched_uuid.uuid4.return_value = forced_uuid

        reply_listener.prepare()
        queue_consumer.prepare()

        queue = reply_listener.queue
        assert queue.name == "rpc.reply-exampleservice-{}".format(forced_uuid)
        assert queue.exchange == exchange
        assert queue.routing_key == forced_uuid

    queue_consumer.register_provider.assert_called_once_with(reply_listener)

    correlation_id = 1
    reply_event = reply_listener.get_reply_event(correlation_id)

    assert reply_listener._reply_events == {1: reply_event}

    message = Mock()
    message.properties.get.return_value = correlation_id
    reply_listener.handle_message("msg", message)

    queue_consumer.ack_message.assert_called_once_with(message)
    assert reply_event.ready()
    assert reply_event.wait() == "msg"

    assert reply_listener._reply_events == {}

    with patch('nameko.rpc._log', autospec=True) as log:
        reply_listener.handle_message("msg", message)
        assert log.debug.call_args == call(
            'Unknown correlation id: %s', correlation_id)


# =============================================================================
# INTEGRATION TESTS
# =============================================================================


def test_rpc_consumer_creates_single_consumer(container_factory, rabbit_config,
                                              rabbit_manager):
    container = container_factory(ExampleService, rabbit_config)
    container.start()

    # we should have 3 queues:
    #   * RPC requests
    #   * RPC replies
    #   * events
    vhost = rabbit_config['vhost']
    queues = rabbit_manager.get_queues(vhost)
    assert len(queues) == 3

    # each one should have one consumer
    rpc_queue = rabbit_manager.get_queue(vhost, "rpc-exampleservice")
    assert len(rpc_queue['consumer_details']) == 1
    evt_queue = rabbit_manager.get_queue(
        vhost, "evt-srcservice-eventtype--exampleservice.async_task")
    assert len(evt_queue['consumer_details']) == 1

    queue_names = [queue['name'] for queue in queues]
    reply_queue_names = [name for name in queue_names if 'rpc.reply' in name]
    assert len(reply_queue_names) == 1
    reply_queue_name = reply_queue_names[0]
    reply_queue = rabbit_manager.get_queue(vhost, reply_queue_name)
    assert len(reply_queue['consumer_details']) == 1

    # and share a single connection
    consumer_connection_names = set(
        queue['consumer_details'][0]['channel_details']['connection_name']
        for queue in [rpc_queue, evt_queue, reply_queue]
    )
    assert len(consumer_connection_names) == 1


def test_rpc_args_kwargs(container_factory, rabbit_config):

    container = container_factory(ExampleService, rabbit_config)
    container.start()

    with entrypoint_hook(container, 'echo') as echo:
        assert echo() == ((), {})
        assert echo("a", "b") == (("a", "b"), {})
        assert echo(foo="bar") == ((), {'foo': 'bar'})
        assert echo("arg", kwarg="kwarg") == (("arg",), {'kwarg': 'kwarg'})


def test_rpc_context_data(container_factory, rabbit_config):

    container = container_factory(ExampleService, rabbit_config)
    container.start()

    context_data = {
        'language': 'en',
        'auth_token': '123456789'
    }

    with entrypoint_hook(container, 'say_hello', context_data) as say_hello:
        assert say_hello() == "hello"

    context_data['language'] = 'fr'

    with entrypoint_hook(container, 'say_hello', context_data) as say_hello:
        assert say_hello() == "bonjour"


@pytest.mark.usefixtures("predictable_call_ids")
def test_rpc_headers(container_factory, rabbit_config):

    container = container_factory(ExampleService, rabbit_config)

    context_data = {
        'language': 'en',
        'bogus_header': '123456789'
    }

    headers = {}
    rpc_consumer = get_dependency(container, RpcConsumer)
    handle_message = rpc_consumer.handle_message

    with patch.object(
            rpc_consumer, 'handle_message', autospec=True) as patched_handler:
        def side_effect(body, message):
            headers.update(message.headers)  # extract message headers
            return handle_message(body, message)

        patched_handler.side_effect = side_effect
        container.start()

    # use a standalone rpc proxy to call exampleservice.say_hello()
    with RpcProxy("exampleservice", rabbit_config, context_data) as proxy:
        proxy.say_hello()

    # bogus_header dropped
    assert headers == {
        'nameko.language': 'en',
        'nameko.call_id_stack': ['standalone_rpc_proxy.call.0'],
    }


@pytest.mark.usefixtures("predictable_call_ids")
def test_rpc_custom_headers(container_factory, rabbit_config):
    container = container_factory(ExampleService, rabbit_config)

    context_data = {
        'language': 'en',
        'bogus_header': '123456789',
        'custom_header': 'specialvalue',
    }

    headers = {}
    rpc_consumer = get_dependency(container, RpcConsumer)
    handle_message = rpc_consumer.handle_message

    with patch.object(
            rpc_consumer, 'handle_message', autospec=True) as patched_handler:
        def side_effect(body, message):
            headers.update(message.headers)  # extract message headers
            return handle_message(body, message)

        patched_handler.side_effect = side_effect
        container.start()

    # use a standalone rpc proxy to call exampleservice.say_hello(),
    # with a worker context that enables "custom_header"
    with RpcProxy("exampleservice", rabbit_config,
                  context_data, CustomWorkerContext) as proxy:
        proxy.say_hello()

    # bogus_header dropped, custom_header present
    assert headers == {
        'nameko.language': 'en',
        'nameko.custom_header': 'specialvalue',
        'nameko.call_id_stack': ['standalone_rpc_proxy.call.0']
    }


def test_rpc_existing_method(container_factory, rabbit_config):

    container = container_factory(ExampleService, rabbit_config)
    container.start()

    with RpcProxy("exampleservice", rabbit_config) as proxy:
        assert proxy.task_a() == "result_a"
        assert proxy.task_b() == "result_b"


def test_rpc_incorrect_signature(container_factory, rabbit_config):

    class Service(object):

        @rpc
        def no_args(self):
            pass

        @rpc
        def args_only(self, a):
            pass

        @rpc
        def kwargs_only(self, a=None):
            pass

        @rpc
        def star_args(self, *args):
            pass

        @rpc
        def star_kwargs(self, **kwargs):
            pass

        @rpc
        def args_star_args(self, a, *args):
            pass

        @rpc
        def args_star_kwargs(self, a, **kwargs):
            pass

    container = container_factory(Service, rabbit_config)
    container.start()

    method_calls = [
        (('no_args', (), {}), True),
        (('no_args', ('bad arg',), {}), False),
        (('args_only', ('arg',), {}), True),
        (('args_only', (), {'a': 'arg'}), True),
        (('args_only', (), {'arg': 'arg'}), False),
        (('kwargs_only', ('a',), {}), True),
        (('kwargs_only', (), {'a': 'arg'}), True),
        (('kwargs_only', (), {'arg': 'arg'}), False),
        (('star_args', ('a', 'b'), {}), True),
        (('star_args', (), {'c': 'c'}), False),
        (('args_star_args', ('a',), {}), True),
        (('args_star_args', ('a', 'b'), {}), True),
        (('args_star_args', (), {}), False),
        (('args_star_args', (), {'c': 'c'}), False),
        (('args_star_kwargs', ('a',), {}), True),
        (('args_star_kwargs', ('a', 'b'), {}), False),
        (('args_star_kwargs', ('a', 'b'), {'c': 'c'}), False),
        (('args_star_kwargs', (), {}), False),
    ]

    for signature, is_valid_call in method_calls:

        method_name, args, kwargs = signature

        with RpcProxy("service", rabbit_config) as proxy:
            method = getattr(proxy, method_name)

            if not is_valid_call:
                with pytest.raises(IncorrectSignature):
                    method(*args, **kwargs)
            else:
                method(*args, **kwargs)  # no raise


def test_rpc_missing_method(container_factory, rabbit_config, rabbit_manager):

    container = container_factory(ExampleService, rabbit_config)
    container.start()

    with RpcProxy("exampleservice", rabbit_config) as proxy:
        with pytest.raises(MethodNotFound) as exc_info:
            proxy.task_c()
    assert exc_info.value.message == "task_c"


def test_rpc_broken_method(container_factory, rabbit_config, rabbit_manager):

    container = container_factory(ExampleService, rabbit_config)
    container.start()

    with RpcProxy("exampleservice", rabbit_config) as proxy:
        with pytest.raises(RemoteError) as exc_info:
            proxy.broken()
    assert exc_info.value.exc_type == "ExampleError"


def test_rpc_unknown_service(container_factory, rabbit_config, rabbit_manager):
    container = container_factory(ExampleService, rabbit_config)
    container.start()

    with RpcProxy("exampleservice", rabbit_config) as proxy:
        # success
        assert proxy.task_a()

        # failure
        with pytest.raises(RemoteError) as exc_info:
            proxy.call_unknown()

    assert exc_info.value.exc_type == "UnknownService"


def test_rpc_unknown_service_standalone(rabbit_config, rabbit_manager):

    with RpcProxy("unknown_service", rabbit_config) as proxy:
        with pytest.raises(UnknownService) as exc_info:
            proxy.anything()

    assert exc_info.value._service_name == 'unknown_service'


def test_rpc_container_being_killed_retries(
        container_factory, rabbit_config, rabbit_manager):

    container = container_factory(ExampleService, rabbit_config)
    container.start()

    def wait_for_result():
        with RpcProxy("exampleservice", rabbit_config) as proxy:
            return proxy.task_a()

    container._being_killed = True

    rpc_provider = get_dependency(container, RpcProvider, name='task_a')

    with patch.object(
        rpc_provider,
        'rpc_consumer',
        wraps=rpc_provider.rpc_consumer,
    ) as wrapped_consumer:
        waiter = eventlet.spawn(wait_for_result)
        with wait_for_call(1, wrapped_consumer.requeue_message):
            pass  # wait until at least one message has been requeued
        assert not waiter.dead

    container._being_killed = False
    assert waiter.wait() == 'result_a'  # now completed


def test_rpc_consumer_sharing(container_factory, rabbit_config,
                              rabbit_manager):
    """ Verify that the RpcConsumer unregisters from the queueconsumer when
    the first provider unregisters itself. Otherwise it keeps consuming
    messages for the unregistered provider, raising MethodNotFound.
    """

    container = container_factory(ExampleService, rabbit_config)
    container.start()

    task_a = get_dependency(container, RpcProvider, name="task_a")
    task_a_stop = task_a.stop

    task_b = get_dependency(container, RpcProvider, name="task_b")
    task_b_stop = task_b.stop

    task_a_stopped = Event()

    def patched_task_a_stop():
        task_a_stop()  # stop immediately
        task_a_stopped.send(True)

    def patched_task_b_stop():
        eventlet.sleep(2)  # stop after 2 seconds
        task_b_stop()

    with patch.object(task_b, 'stop', patched_task_b_stop), \
            patch.object(task_a, 'stop', patched_task_a_stop):

        # stop the container and wait for task_a to stop
        # task_b will still be in the process of stopping
        eventlet.spawn(container.stop)
        task_a_stopped.wait()

        # try to call task_a.
        # should timeout, rather than raising MethodNotFound
        with RpcProxy("exampleservice", rabbit_config) as proxy:
            with pytest.raises(eventlet.Timeout):
                with eventlet.Timeout(1):
                    proxy.task_a()

    # kill the container so we don't have to wait for task_b to stop
    container.kill()


def test_rpc_consumer_cannot_exit_with_providers(
        container_factory, rabbit_config, rabbit_manager):

    container = container_factory(ExampleService, rabbit_config)
    container.start()

    task_a = get_dependency(container, RpcProvider, name="task_a")

    def never_stops():
        while True:
            eventlet.sleep()

    with patch.object(task_a, 'stop', never_stops):
        with pytest.raises(eventlet.Timeout):
            with eventlet.Timeout(1):
                container.stop()

    # kill off task_a's misbehaving rpc provider
    container.kill()



""" Service unit testing best practice.
"""

from nameko.rpc import rpc_proxy, rpc
from nameko.testing.services import worker_factory


class ConversionService(object):
    math = rpc_proxy("math_service")

    @rpc
    def inches_to_cm(self, inches):
        return self.math.multiply(inches, 2.54)

    @rpc
    def cms_to_inches(self, cms):
        return self.math.divide(cms, 2.54)

# =============================================================================
# Begin test
# =============================================================================


def test_conversion_service():
    # create instance
    # dependencies are replaced with mocks unless otherwise given - see
    # :class:`nameko.testing.services.worker_factory`
    service = worker_factory(ConversionService)

    # replace "math" service
    service.math.multiply.side_effect = lambda x, y: x * y
    service.math.divide.side_effect = lambda x, y: x / y

    # test inches_to_cm business logic
    assert service.inches_to_cm(300) == 762
    service.math.multiply.assert_called_once_with(300, 2.54)

    # test cms_to_inches business logic
    assert service.cms_to_inches(762) == 300
    service.math.divide.assert_called_once_with(762, 2.54)



import pytest

from nameko.containers import ServiceContainer, WorkerContext
from nameko.constants import (
    AUTH_TOKEN_CONTEXT_KEY, LANGUAGE_CONTEXT_KEY, USER_ID_CONTEXT_KEY)
from nameko.contextdata import (
    language, user_id, auth_token, ContextDataProvider)
from nameko.dependencies import DependencyFactory, injection
from nameko.testing.utils import get_dependency

CUSTOM_CONTEXT_KEY = "custom"


@injection
def custom_value():
    return DependencyFactory(ContextDataProvider, CUSTOM_CONTEXT_KEY)


class CustomWorkerContext(WorkerContext):
    context_keys = WorkerContext.context_keys + (CUSTOM_CONTEXT_KEY,)


class Service(object):

    # builtin context data providers
    auth_token = auth_token()
    language = language()
    user_id = user_id()

    # custom context data provider
    custom_value = custom_value()


@pytest.fixture
def container():
    return ServiceContainer(Service, CustomWorkerContext, {})


def test_get_custom_context_value(container):
    provider = get_dependency(
        container, ContextDataProvider, name="custom_value")
    worker_ctx = WorkerContext(
        container, "service", provider, data={CUSTOM_CONTEXT_KEY: "hello"})

    assert provider.acquire_injection(worker_ctx) == "hello"


def test_get_unset_value(container):
    provider = get_dependency(
        container, ContextDataProvider, name="custom_value")
    worker_ctx = WorkerContext(
        container, "service", provider, data={})

    assert provider.acquire_injection(worker_ctx) is None


@pytest.mark.parametrize('provider_name, context_key', [
    ('auth_token', AUTH_TOKEN_CONTEXT_KEY),
    ('language', LANGUAGE_CONTEXT_KEY),
    ('user_id', USER_ID_CONTEXT_KEY),

])
def test_get_builtin_providers(provider_name, context_key, container):
    provider = get_dependency(
        container, ContextDataProvider, name=provider_name)
    worker_ctx = WorkerContext(
        container, "service", provider, data={context_key: 'value'})

    assert provider.acquire_injection(worker_ctx) == "value"



from __future__ import absolute_import

from kombu import Connection
from kombu.common import itermessages, maybe_declare

from nameko.containers import WorkerContext
from nameko.rpc import ServiceProxy, ReplyListener


class ConsumeEvent(object):
    """ Event for the RPC consumer with the same interface as eventlet.Event.
    """
    def __init__(self, queue_consumer, correlation_id):
        self.correlation_id = correlation_id
        self.queue_consumer = queue_consumer

    def send(self, body):
        self.body = body

    def wait(self):
        """ Makes a blocking call to its queue_consumer until the message
        with the given correlation_id has been processed.

        By the time the blocking call exits, self.send() will have been called
        with the body of the received message
        (see :class:nameko.rpc.ReplyListener.handle_message).

        Exceptions are raised directly.
        """
        self.queue_consumer.poll_messages(self.correlation_id)
        return self.body


class PollingQueueConsumer(object):
    """ Implements a minimum interface of the
    :class:`~messaging.QueueConsumer`. Instead of processing messages in a
    separate thread it provides a polling method to block until a message with
    the same correlation ID of the RPC-proxy call arrives.
    """
    def register_provider(self, provider):
        self.provider = provider
        self.connection = Connection(provider.container.config['AMQP_URI'])
        self.channel = self.connection.channel()
        self.queue = provider.queue
        maybe_declare(self.queue, self.channel)

    def unregister_provider(self, provider):
        self.connection.close()

    def ack_message(self, msg):
        msg.ack()

    def poll_messages(self, correlation_id):
        channel = self.channel
        conn = channel.connection

        for body, msg in itermessages(conn, channel, self.queue, limit=None):
            if correlation_id == msg.properties.get('correlation_id'):
                self.provider.handle_message(body, msg)
                break


class SingleThreadedReplyListener(ReplyListener):
    """ A ReplyListener which uses a custom queue consumer and ConsumeEvent.
    """
    queue_consumer = None

    def __init__(self):
        self.queue_consumer = PollingQueueConsumer()
        super(SingleThreadedReplyListener, self).__init__()

    def get_reply_event(self, correlation_id):
        reply_event = ConsumeEvent(self.queue_consumer, correlation_id)
        self._reply_events[correlation_id] = reply_event
        return reply_event


class RpcProxy(object):
    """
    A single-threaded RPC proxy to a named service. Method calls on the
    proxy are converted into RPC calls to the service, with responses
    returned directly.

    Enables services not hosted by nameko to make RPC requests to a nameko
    cluster. It is commonly used as a context manager but may also be manually
    started and stopped.

    *Usage*

    As a context manager::

        with RpcProxy('targetservice', config) as proxy:
            proxy.method()

    The equivalent call, manually starting and stopping::

        targetservice_proxy = RpcProxy('targetservice', config)
        proxy = targetservice_proxy.start()
        proxy.method()
        targetservice_proxy.stop()

    If you call ``start()`` you must eventually call ``stop()`` to close the
    connection to the broker.

    You may also supply ``context_data``, a dictionary of data to be
    serialised into the AMQP message headers, and specify custom worker
    context class to serialise them.
    """
    class ServiceContainer(object):
        """ Implements a minimum interface of the
        :class:`~containers.ServiceContainer` to be used by the subclasses
        and rpc imports in this module.
        """
        service_name = "standalone_rpc_proxy"

        def __init__(self, config):
            self.config = config

    class DummyProvider(object):
        name = "call"

    def __init__(self, container_service_name, config, context_data=None,
                 worker_ctx_cls=WorkerContext):

        container = RpcProxy.ServiceContainer(config)

        reply_listener = SingleThreadedReplyListener()
        reply_listener.container = container

        worker_ctx = worker_ctx_cls(
            container, service=None, provider=self.DummyProvider,
            data=context_data)
        service_proxy = ServiceProxy(worker_ctx, container_service_name,
                                     reply_listener)

        self._reply_listener = reply_listener
        self._service_proxy = service_proxy

    def __enter__(self):
        self.start()
        return self._service_proxy

    def __exit__(self, tpe, value, traceback):
        self.stop()

    def start(self):
        self._reply_listener.prepare()
        return self._service_proxy

    def stop(self):
        self._reply_listener.stop()



from __future__ import absolute_import
from contextlib import contextmanager
from functools import partial

from logging import getLogger

from concurrent import futures
import greenlet
from nameko.utils import try_wraps
from nameko.dependencies import InjectionProvider, injection, DependencyFactory


_log = getLogger(__name__)


class ParallelProxyFactory(object):
    def __init__(self, thread_provider):
        self.container = thread_provider

    @contextmanager
    def __call__(self, to_wrap):
        executor = ParallelExecutor(self.container)
        proxy = ParallelProxy(executor, to_wrap)
        with executor:
            yield proxy


class ParallelExecutor(futures.Executor):
    def __init__(self, container):
        self.container = container
        self._spawned_threads = set()
        self._shutdown = False

    def submit(self, func, *args, **kwargs):
        if self._shutdown:
            raise RuntimeError('cannot schedule new futures after '
                               'shutdown')

        f = futures.Future()

        @try_wraps(func)
        def do_function_call():
            # A failure on the function call doesn't propagate up and kill
            # the container: it's marked on the future only.
            try:
                result = func(*args, **kwargs)
            except Exception as exc:
                self._handle_call_complete(f, exc=exc)
                return None, exc
            else:
                self._handle_call_complete(f, result=result)
                return result, None

        gt = self.container.spawn_managed_thread(do_function_call)
        self._spawned_threads.add(gt)
        gt.link(partial(self._handle_thread_exited, f))

        # Mark the future as running immediately. It's not possible for it
        # to have been cancelled.
        f.set_running_or_notify_cancel()

        return f

    def _handle_call_complete(self, future, result=None, exc=None):
        """
        Set a result or exception on ``future``

        Called when the future's function completes, inside the spawned thread.
        """
        if exc:
            future.set_exception(exc)
        else:
            future.set_result(result)

    def _handle_thread_exited(self, future, gt):
        """
        Handle the completion of a thread spawned for a future.
        """
        self._spawned_threads.remove(gt)
        try:
            gt.wait()
        except greenlet.GreenletExit as green_exit:
            # 'Normal' errors with the submitted function call are handled
            # by `_handle_call_complete`. This only occurs when the
            # container is killed before the future's thread exits.
            future.set_exception(green_exit)

    def shutdown(self, wait=True):
        """
        Call to ensure all spawned threads have finished.

        This method is called when automatically ParallelExecutor is used as a
        Context Manager
        """
        self._shutdown = True
        if wait:
            # Fix the list of threads to wait for, so threads that remove while
            # waiting don't change the iterator
            for thread in list(self._spawned_threads):
                thread.wait()


class ParallelProxy(object):
    def __init__(self, executor, to_wrap=None):
        """
        Create a new proxy around an object then ensures method calls are ran
        by the executor.

        Attribute access and function calls on the wrapped object can be
        performed as normal, but writing to fields is not allowed.

        You can use this as a context manager: it uses the associated executor
        as one.
        """
        self.__dict__['executor'] = executor
        self.__dict__['to_wrap'] = to_wrap

    def __getattr__(self, name):
        """
        Callables accessed on the wrapped object are performed by the executor
        calling `submit`
        """
        wrapped_attribute = getattr(self.to_wrap, name)
        if callable(wrapped_attribute):
            # Give name/docs of wrapped_attribute to do_submit
            @try_wraps(wrapped_attribute)
            def do_submit(*args, **kwargs):
                return self.executor.submit(wrapped_attribute, *args, **kwargs)
            return do_submit
        return wrapped_attribute

    def __setattr__(self, key, value):
        """
        Attribute setting is unsupported on proxy objects
        """
        raise ProxySettingUnsupportedException()


class ProxySettingUnsupportedException(Exception):
    pass


class ParallelProvider(InjectionProvider):
    def acquire_injection(self, worker_ctx):
        return ParallelProxyFactory(self.container)


@injection
def parallel_provider(*args, **kwargs):
    return DependencyFactory(ParallelProvider, *args, **kwargs)



import pytest

from nameko.nameko_doc.processor import ServiceDocProcessor


def empty_loader():
    return []


def empty_event_loader(_):
    return []


class TestProcessor(object):
    def test_build_empty_project(self, temp_folder_factory):
        output_folder = temp_folder_factory(reserve_only=True)

        processor = ServiceDocProcessor(
            output_folder, empty_loader, empty_event_loader
        )
        processor.write_docs()

        assert output_folder.exists()
        assert output_folder.files() == []

    def test_build_into_existing_folder(self, temp_folder_factory):
        output_folder = temp_folder_factory()

        # Add something in there
        output_folder.joinpath('bye.txt').write_text('bye!')

        assert len(output_folder.files()) == 1

        processor = ServiceDocProcessor(
            output_folder, empty_loader, empty_event_loader
        )

        with pytest.raises(ValueError):
            processor.write_docs()
        assert len(output_folder.files()) == 1



import eventlet
import pytest

from nameko.events import event_handler, Event, BROADCAST
from nameko.standalone.events import event_dispatcher
from nameko.standalone.rpc import RpcProxy
from nameko.rpc import rpc
from nameko.runners import ServiceRunner, run_services
from nameko.testing.utils import assert_stops_raising, get_container


class TestService1(object):
    name = 'foobar_1'


class TestService2(object):
    name = 'foobar_2'


received = []


@pytest.fixture(autouse=True)
def reset_mock():
    del received[:]


class TestEvent(Event):
    type = "testevent"


class Service(object):

    @rpc
    @event_handler("srcservice", TestEvent.type, handler_type=BROADCAST,
                   reliable_delivery=False)
    def handle(self, msg):
        received.append(msg)


def test_runner_lifecycle():
    events = set()

    class Container(object):
        def __init__(self, service_cls, worker_ctx_cls, config):
            self.service_name = service_cls.__name__
            self.service_cls = service_cls
            self.worker_ctx_cls = worker_ctx_cls

        def start(self):
            events.add(('start', self.service_cls.name, self.service_cls))

        def stop(self):
            events.add(('stop', self.service_cls.name, self.service_cls))

        def kill(self):
            events.add(('kill', self.service_cls.name, self.service_cls))

        def wait(self):
            events.add(('wait', self.service_cls.name, self.service_cls))

    config = {}
    runner = ServiceRunner(config, container_cls=Container)

    runner.add_service(TestService1)
    runner.add_service(TestService2)

    runner.start()

    assert events == {
        ('start', 'foobar_1', TestService1),
        ('start', 'foobar_2', TestService2),
    }

    events = set()
    runner.stop()
    assert events == {
        ('stop', 'foobar_1', TestService1),
        ('stop', 'foobar_2', TestService2),
    }

    events = set()
    runner.kill()
    assert events == {
        ('kill', 'foobar_1', TestService1),
        ('kill', 'foobar_2', TestService2),
    }

    events = set()
    runner.wait()
    assert events == {
        ('wait', 'foobar_1', TestService1),
        ('wait', 'foobar_2', TestService2),
    }


def test_contextual_lifecycle():
    events = set()

    class Container(object):
        def __init__(self, service_cls, worker_ctx_cls, config):
            self.service_name = service_cls.__name__
            self.service_cls = service_cls
            self.worker_ctx_cls = worker_ctx_cls

        def start(self):
            events.add(('start', self.service_cls.name, self.service_cls))

        def stop(self):
            events.add(('stop', self.service_cls.name, self.service_cls))

        def kill(self, exc=None):
            events.add(('kill', self.service_cls.name, self.service_cls))

        def wait(self):
            events.add(('wait', self.service_cls.name, self.service_cls))

    config = {}

    with run_services(config, TestService1, TestService2,
                      container_cls=Container):
        # Ensure the services were started
        assert events == {
            ('start', 'foobar_1', TestService1),
            ('start', 'foobar_2', TestService2),
        }

    # ...and that they were stopped
    assert events == {
        ('start', 'foobar_1', TestService1),
        ('start', 'foobar_2', TestService2),
        ('stop', 'foobar_1', TestService1),
        ('stop', 'foobar_2', TestService2),
    }

    events = set()
    with run_services(config, TestService1, TestService2,
                      container_cls=Container, kill_on_exit=True):
        # Ensure the services were started
        assert events == {
            ('start', 'foobar_1', TestService1),
            ('start', 'foobar_2', TestService2),
        }

    # ...and that they were killed
    assert events == {
        ('kill', 'foobar_1', TestService1),
        ('kill', 'foobar_2', TestService2),
        ('start', 'foobar_1', TestService1),
        ('start', 'foobar_2', TestService2),
    }


def test_runner_waits_raises_error():
    class Container(object):
        def __init__(self, service_cls, worker_ctx_cls, config):
            pass

        def start(self):
            pass

        def stop(self):
            pass

        def kill(self, exc):
            pass

        def wait(self):
            raise Exception('error in container')

    runner = ServiceRunner(config={}, container_cls=Container)
    runner.add_service(TestService1)
    runner.start()

    with pytest.raises(Exception) as exc_info:
        runner.wait()
    assert exc_info.value.args == ('error in container',)


def test_multiple_runners_coexist(runner_factory, rabbit_config,
                                  rabbit_manager):

    runner1 = runner_factory(rabbit_config, Service)
    runner1.start()

    runner2 = runner_factory(rabbit_config, Service)
    runner2.start()

    vhost = rabbit_config['vhost']

    # verify there are two event queues with a single consumer each
    def check_consumers():
        evt_queues = [queue for queue in rabbit_manager.get_queues(vhost)
                      if queue['name'].startswith('evt-srcservice-testevent')]
        assert len(evt_queues) == 2
        for queue in evt_queues:
            assert queue['consumers'] == 1

    # rabbit's management API seems to lag
    assert_stops_raising(check_consumers)

    # test events (both services will receive if in "broadcast" mode)
    event_data = "msg"
    with event_dispatcher('srcservice', rabbit_config) as dispatch:
        dispatch(TestEvent(event_data))

    with eventlet.Timeout(1):
        while len(received) < 2:
            eventlet.sleep()

        assert received == [event_data, event_data]

    # verify there are two consumers on the rpc queue
    rpc_queue = rabbit_manager.get_queue(vhost, 'rpc-service')
    assert rpc_queue['consumers'] == 2

    # test rpc (only one service will respond)
    del received[:]
    arg = "msg"
    with RpcProxy('service', rabbit_config) as proxy:
        proxy.handle(arg)

    with eventlet.Timeout(1):
        while len(received) == 0:
            eventlet.sleep()

        assert received == [arg]


def test_runner_with_duplicate_services(runner_factory, rabbit_config):

    # host Service multiple times
    runner = runner_factory(rabbit_config)
    runner.add_service(Service)
    runner.add_service(Service)  # no-op
    runner.start()

    # it should only be hosted once
    assert len(runner.containers) == 1

    # test events (only one service is hosted)
    event_data = "msg"
    with event_dispatcher('srcservice', rabbit_config) as dispatch:
        dispatch(TestEvent(event_data))

    with eventlet.Timeout(1):
        while len(received) == 0:
            eventlet.sleep()

        assert received == [event_data]

    # test rpc
    arg = "msg"
    del received[:]

    with RpcProxy("service", rabbit_config) as proxy:
        proxy.handle(arg)

    with eventlet.Timeout(1):
        while len(received) == 0:
            eventlet.sleep()

        assert received == [arg]


def test_runner_catches_managed_thread_errors(runner_factory, rabbit_config):

    class Broken(Exception):
        pass

    def raises():
        raise Broken('error')

    runner = runner_factory(rabbit_config, Service)

    container = get_container(runner, Service)
    container.spawn_managed_thread(raises)

    with pytest.raises(Broken):
        runner.wait()



""" Service integration testing best practice.
"""

from nameko.rpc import rpc, rpc_proxy
from nameko.runners import ServiceRunner
from nameko.testing.utils import get_container
from nameko.testing.services import entrypoint_hook


class ServiceX(object):
    name = "service_x"

    y = rpc_proxy("service_y")

    @rpc
    def remote_method(self, value):
        res = "{}-x".format(value)
        return self.y.append_identifier(res)


class ServiceY(object):
    name = "service_y"

    @rpc
    def append_identifier(self, value):
        return "{}-y".format(value)

# =============================================================================
# Begin test
# =============================================================================

def test_service_integration():
    config = {'AMQP_URI': 'amqp://guest:guest@localhost:5672/'}
    runner = ServiceRunner(config)
    runner.add_service(ServiceX)
    runner.add_service(ServiceY)
    runner.start()

    container = get_container(runner, ServiceX)

    with entrypoint_hook(container, "remote_method") as entrypoint:
        assert entrypoint("value") == "value-x-y"

    runner.stop()



import eventlet
eventlet.monkey_patch()

import itertools
import logging
import sys
from urlparse import urlparse

from kombu import pools
from mock import patch
import pytest

from nameko.containers import ServiceContainer, WorkerContext
from nameko.runners import ServiceRunner
from nameko.testing.utils import (
    get_rabbit_manager, reset_rabbit_vhost, reset_rabbit_connections,
    get_rabbit_connections)


def pytest_addoption(parser):
    parser.addoption(
        '--blocking-detection',
        action='store_true',
        dest='blocking_detection',
        default=False,
        help='turn on eventlet hub blocking detection')

    parser.addoption(
        "--log-level", action="store",
        default='DEBUG',
        help=("The logging-level for the test run."))

    parser.addoption(
        "--amqp-uri", action="store", dest='AMQP_URI',
        default='amqp://guest:guest@localhost:5672/nameko',
        help=("The AMQP-URI to connect to rabbit with."))

    parser.addoption(
        "--rabbit-ctl-uri", action="store", dest='RABBIT_CTL_URI',
        default='http://guest:guest@localhost:15672',
        help=("The URI for rabbit's management API."))


def pytest_configure(config):
    # monkey patch an encoding attribute onto GreenPipe to
    # satisfy a pytest assertion
    import py
    from eventlet.greenio import GreenPipe
    GreenPipe.encoding = py.std.sys.stdout.encoding

    if config.option.blocking_detection:
        from eventlet import debug
        debug.hub_blocking_detection(True)

    log_level = config.getoption('log_level')
    if log_level is not None:
        log_level = getattr(logging, log_level)
        logging.basicConfig(level=log_level, stream=sys.stderr)


@pytest.fixture
def empty_config(request):
    return {}


@pytest.fixture(scope='session')
def rabbit_manager(request):
    config = request.config
    return get_rabbit_manager(config.getoption('RABBIT_CTL_URI'))


@pytest.yield_fixture()
def rabbit_config(request, rabbit_manager):
    amqp_uri = request.config.getoption('AMQP_URI')

    conf = {'AMQP_URI': amqp_uri}

    uri = urlparse(amqp_uri)
    vhost = uri.path[1:].replace('/', '%2F')
    username = uri.username

    conf['vhost'] = vhost
    conf['username'] = username

    reset_rabbit_connections(vhost, rabbit_manager)
    reset_rabbit_vhost(vhost, username, rabbit_manager)

    yield conf

    pools.reset()  # close connections in pools

    # raise a runtime error if the test leaves any connections lying around
    connections = get_rabbit_connections(vhost, rabbit_manager)
    if connections:
        count = len(connections)
        raise RuntimeError("{} rabbit connection(s) left open.".format(count))


@pytest.yield_fixture
def container_factory(rabbit_config):

    all_containers = []

    def make_container(service_cls, config, worker_ctx_cls=None):
        if worker_ctx_cls is None:
            worker_ctx_cls = WorkerContext

        container = ServiceContainer(service_cls, worker_ctx_cls, config)
        all_containers.append(container)
        return container

    yield make_container

    for c in all_containers:
        try:
            c.stop()
        except:
            pass


@pytest.yield_fixture
def runner_factory(rabbit_config):

    all_runners = []

    def make_runner(config, *service_classes):
        runner = ServiceRunner(config)
        for service_cls in service_classes:
            runner.add_service(service_cls)
        all_runners.append(runner)
        return runner

    yield make_runner

    for r in all_runners:
        try:
            r.stop()
        except:
            pass


@pytest.yield_fixture
def predictable_call_ids(request):
    with patch('nameko.containers.new_call_id', autospec=True) as get_id:
        get_id.side_effect = (str(i) for i in itertools.count())
        yield get_id



# Nameko relies on eventlet
# You should monkey patch the standard library as early as possible to avoid
# importing anything before the patch is applied.
# See http://eventlet.net/doc/patching.html#monkeypatching-the-standard-library
import eventlet
eventlet.monkey_patch()

import logging
logger = logging.getLogger(__name__)

import random

from nameko.rpc import rpc_proxy
from nameko.runners import ServiceRunner
from nameko.timer import timer


class RpcClient(object):

    adder = rpc_proxy('adderservice')

    @timer(interval=2)
    def add(self):
        x = random.randint(0, 10)
        y = random.randint(0, 10)
        res = self.adder.add(x, y)
        logger.info("{} + {} = {}".format(x, y, res))


def main():

    logging.basicConfig(level=logging.DEBUG)

    config = {'AMQP_URI': 'amqp://guest:guest@localhost:5672/'}
    runner = ServiceRunner(config)
    runner.add_service(RpcClient)
    runner.start()

    try:
        runner.wait()
    except KeyboardInterrupt:
        runner.stop()

if __name__ == '__main__':
    main()



CALL_ID_STACK_CONTEXT_KEY = 'call_id_stack'
AUTH_TOKEN_CONTEXT_KEY = 'auth_token'
LANGUAGE_CONTEXT_KEY = 'language'
USER_ID_CONTEXT_KEY = 'user_id'


NAMEKO_CONTEXT_KEYS = (
    LANGUAGE_CONTEXT_KEY,
    USER_ID_CONTEXT_KEY,
    AUTH_TOKEN_CONTEXT_KEY,
    CALL_ID_STACK_CONTEXT_KEY,
)


MAX_WORKERS_CONFIG_KEY = 'max_workers'
PARENT_CALLS_CONFIG_KEY = 'parent_calls_tracked'

DEFAULT_MAX_WORKERS = 10
DEFAULT_PARENT_CALLS_TRACKED = 10

DEFAULT_RETRY_POLICY = {'max_retries': 3}



# Nameko relies on eventlet
# You should monkey patch the standard library as early as possible to avoid
# importing anything before the patch is applied.
# See http://eventlet.net/doc/patching.html#monkeypatching-the-standard-library
import eventlet
eventlet.monkey_patch()

import os
import tempfile

from kombu import Exchange, Queue

from nameko.dependencies import InjectionProvider, injection, DependencyFactory
from nameko.messaging import consume
from nameko.runners import ServiceRunner

demo_ex = Exchange('demo_ex', durable=False, auto_delete=True)
demo_queue = Queue('demo_queue', exchange=demo_ex, durable=False,
                   auto_delete=True)


class InvalidPath(Exception):
    pass


class LogFile(InjectionProvider):
    """ Developer docs for LogFile InjectionProvider
    """
    def __init__(self, path):
        if path is None:
            path = os.path.join(tempfile.mkdtemp(), 'nameko.log')
        self.path = path

    def prepare(self):
        self.file_handle = open(self.path, 'w')

    def stop(self):
        self.file_handle.close()

    def acquire_injection(self, worker_ctx):
        def log(msg):
            self.file_handle.write(msg + "\n")
        return log

    def worker_teardown(self, worker_ctx):
        self.file_handle.flush()


@injection
def file_logger(path=None):
    """ User docs for file logger
    """
    if path is not None:
        check_path = path
        if not os.path.exists(check_path):
            check_path = os.path.dirname(path)
        if not os.access(check_path, os.W_OK):
            raise InvalidPath("File or dir not writable: {}".format(path))

    return DependencyFactory(LogFile, path)


class MessagingConsumer(object):

    log = file_logger('/tmp/nameko.log')

    @consume(demo_queue)
    def process(self, payload):
        self.log(payload)


def main():
    import logging
    logging.basicConfig(level=logging.DEBUG)

    config = {'AMQP_URI': 'amqp://guest:guest@localhost:5672/'}
    runner = ServiceRunner(config)
    runner.add_service(MessagingConsumer)
    runner.start()

    try:
        runner.wait()
    except KeyboardInterrupt:
        runner.stop()

if __name__ == '__main__':
    main()
